@@ -34,7 +34,7 @@ void cdfDefRecord(stream_t *streamptr);
 
 void cdfCopyRecord(stream_t *streamptr2, stream_t *streamptr1);
 
-void cdfDefineAttributes(int vlistID, int varID, int fileID, int ncvarID);
+void cdfDefineAttributes(int filetype, int vlistID, int varID, int fileID, int ncvarID);
 
 void cdf_read_record(stream_t *streamptr, int memtype, void *data, size_t *nmiss);
 void cdf_write_record(stream_t *streamptr, int memtype, const void *data, size_t nmiss);
