@@ -11,7 +11,6 @@
 #include "stream_scan.h"
 #include "stream_ext.h"
 #include "varscan.h"
-#include "datetime.h"
 #include "extra.h"
 #include "get_num_missvals.h"
 #include "exse.h"
@@ -79,14 +78,14 @@ int extInqRecord(stream_t *streamptr, int *varID, int *levelID)
 static void
 ext_read_recordSP(stream_t *streamptr, float *data, size_t *nmiss)
 {
-  const int vlistID = streamptr->vlistID;
-  const int fileID = streamptr->fileID;
-  const int tsID = streamptr->curTsID;
+  int vlistID = streamptr->vlistID;
+  int fileID = streamptr->fileID;
+  int tsID = streamptr->curTsID;
 
-  const int vrecID = streamptr->tsteps[tsID].curRecID;
-  const int recID = streamptr->tsteps[tsID].recIDs[vrecID];
-  const int varID = streamptr->tsteps[tsID].records[recID].varID;
-  const off_t recpos = streamptr->tsteps[tsID].records[recID].position;
+  int vrecID = streamptr->tsteps[tsID].curRecID;
+  int recID = streamptr->tsteps[tsID].recIDs[vrecID];
+  int varID = streamptr->tsteps[tsID].records[recID].varID;
+  off_t recpos = streamptr->tsteps[tsID].records[recID].position;
 
   fileSetPos(fileID, recpos, SEEK_SET);
 
@@ -97,8 +96,8 @@ ext_read_recordSP(stream_t *streamptr, float *data, size_t *nmiss)
   extInqHeader(extp, header);
   extInqDataSP(extp, data);
 
-  const double missval = vlistInqVarMissval(vlistID, varID);
-  const size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
+  double missval = vlistInqVarMissval(vlistID, varID);
+  size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
 
   if (vlistInqVarNumber(vlistID, varID) == CDI_REAL)
     *nmiss = get_num_missvalsSP(size, data, missval);
@@ -111,14 +110,14 @@ ext_read_recordSP(stream_t *streamptr, float *data, size_t *nmiss)
 static void
 ext_read_recordDP(stream_t *streamptr, double *data, size_t *nmiss)
 {
-  const int vlistID = streamptr->vlistID;
-  const int fileID = streamptr->fileID;
-  const int tsID = streamptr->curTsID;
+  int vlistID = streamptr->vlistID;
+  int fileID = streamptr->fileID;
+  int tsID = streamptr->curTsID;
 
-  const int vrecID = streamptr->tsteps[tsID].curRecID;
-  const int recID = streamptr->tsteps[tsID].recIDs[vrecID];
-  const int varID = streamptr->tsteps[tsID].records[recID].varID;
-  const off_t recpos = streamptr->tsteps[tsID].records[recID].position;
+  int vrecID = streamptr->tsteps[tsID].curRecID;
+  int recID = streamptr->tsteps[tsID].recIDs[vrecID];
+  int varID = streamptr->tsteps[tsID].records[recID].varID;
+  off_t recpos = streamptr->tsteps[tsID].records[recID].position;
 
   fileSetPos(fileID, recpos, SEEK_SET);
 
@@ -129,8 +128,8 @@ ext_read_recordDP(stream_t *streamptr, double *data, size_t *nmiss)
   extInqHeader(extp, header);
   extInqDataDP(extp, data);
 
-  const double missval = vlistInqVarMissval(vlistID, varID);
-  const size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
+  double missval = vlistInqVarMissval(vlistID, varID);
+  size_t size = gridInqSize(vlistInqVarGrid(vlistID, varID));
 
   if (vlistInqVarNumber(vlistID, varID) == CDI_REAL)
     *nmiss = get_num_missvalsDP(size, data, missval);
@@ -167,7 +166,7 @@ extDefRecord(stream_t *streamptr)
   header[0] = record->vdate;
   header[1] = pnum;
   header[2] = record->ilevel;
-  const int gridID = record->gridID;
+  int gridID = record->gridID;
   cdi_check_gridsize_int_limit("EXTRA", gridInqSize(gridID));
   header[3] = (int) gridInqSize(gridID);
 
@@ -204,9 +203,9 @@ ext_write_record(stream_t *streamptr, int memtype, const void *data)
 static void
 extAddRecord(stream_t *streamptr, int param, int level, size_t xysize, size_t recsize, off_t position, int prec, int number)
 {
-  const int vlistID = streamptr->vlistID;
-  const int tsID = streamptr->curTsID;
-  const int recID = recordNewEntry(streamptr, tsID);
+  int vlistID = streamptr->vlistID;
+  int tsID = streamptr->curTsID;
+  int recID = recordNewEntry(streamptr, tsID);
   record_t *record = &streamptr->tsteps[tsID].records[recID];
 
   record->size = recsize;
@@ -220,11 +219,15 @@ extAddRecord(stream_t *streamptr, int param, int level, size_t xysize, size_t re
   grid->x.size = xysize;
   grid->y.size = 0;
   struct addIfNewRes gridAdded = cdiVlistAddGridIfNew(vlistID, grid, 0);
-  const int gridID = gridAdded.Id;
-  if (!gridAdded.isNew) Free(grid);
+  int gridID = gridAdded.Id;
+  if (!gridAdded.isNew)
+    {
+      grid_free(grid);
+      Free(grid);
+    }
 
-  const int leveltype = ZAXIS_GENERIC;
-  const int datatype = extInqDatatype(prec, number);
+  int leveltype = ZAXIS_GENERIC;
+  int datatype = extInqDatatype(prec, number);
 
   int varID, levelID = 0;
   varAddRecord(recID, param, gridID, leveltype, 0, level, 0, 0, 0, datatype, &varID, &levelID, TSTEP_INSTANT, 0, -1, NULL, NULL,
@@ -242,8 +245,9 @@ extAddRecord(stream_t *streamptr, int param, int level, size_t xysize, size_t re
 static void
 extScanTimestep1(stream_t *streamptr)
 {
+  CdiDateTime datetime0;
+  cdiDateTime_init(&datetime0);
   int header[4];
-  CmpDateTime datetime0 = { INT_MIN, INT_MIN };
   off_t recpos = 0;
   extrec_t *extp = (extrec_t *) streamptr->record->objectp;
 
@@ -253,7 +257,7 @@ extScanTimestep1(stream_t *streamptr)
   if (tsID != 0) Error("Internal problem! tstepsNewEntry returns %d", tsID);
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
-  const int fileID = streamptr->fileID;
+  int fileID = streamptr->fileID;
 
   int nrecs = 0;
   while (true)
@@ -265,22 +269,22 @@ extScanTimestep1(stream_t *streamptr)
           break;
         }
 
-      const size_t recsize = (size_t) (fileGetPos(fileID) - recpos);
+      size_t recsize = (size_t) (fileGetPos(fileID) - recpos);
 
       extInqHeader(extp, header);
 
-      const int vdate = header[0];
-      const int vtime = 0;
-      const int rcode = header[1];
-      const int rlevel = header[2];
-      const int rxysize = header[3];
-      const int param = cdiEncodeParam(rcode, 255, 255);
-      CmpDateTime datetime = { .date = vdate, .time = vtime };
+      int vdate = header[0];
+      int vtime = 0;
+      int rcode = header[1];
+      int rlevel = header[2];
+      int rxysize = header[3];
+      int param = cdiEncodeParam(rcode, 255, 255);
+      CdiDateTime datetime = cdiDateTime_set(vdate, vtime);
 
       if (nrecs == 0)
         {
           datetime0 = datetime;
-          taxis->vdatetime = cdiDateTime_set(vdate, vtime);
+          taxis->vDateTime = datetime;
         }
       else
         {
@@ -288,7 +292,7 @@ extScanTimestep1(stream_t *streamptr)
           for (int recID = 0; recID < nrecs; recID++)
             if (param == records[recID].param && rlevel == records[recID].ilevel) goto tstepScanLoopFinished;
 
-          if (datetime_differ(datetime, datetime0)) Warning("Inconsistent verification time for code %d level %d", rcode, rlevel);
+          if (cdiDateTime_isNE(datetime, datetime0)) Warning("Inconsistent verification time for code %d level %d", rcode, rlevel);
         }
 
       nrecs++;
@@ -303,11 +307,11 @@ tstepScanLoopFinished:
 
   cdi_generate_vars(streamptr);
 
-  const int taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
   taxis->type = TAXIS_ABSOLUTE;
-  taxis->rdatetime = taxis->vdatetime;
+  taxis->rDateTime = taxis->vDateTime;
 
-  const int vlistID = streamptr->vlistID;
+  int vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
 
   vlist_check_contents(vlistID);
@@ -327,8 +331,8 @@ extScanTimestep2(stream_t *streamptr)
 
   streamptr->curTsID = 1;
 
-  const int fileID = streamptr->fileID;
-  const int vlistID = streamptr->vlistID;
+  int fileID = streamptr->fileID;
+  int vlistID = streamptr->vlistID;
 
   int tsID = streamptr->rtsteps;
   if (tsID != 1) Error("Internal problem! unexpected timestep %d", tsID + 1);
@@ -340,7 +344,7 @@ extScanTimestep2(stream_t *streamptr)
   cdi_create_records(streamptr, tsID);
   record_t *records = streamptr->tsteps[tsID].records;
 
-  const int nrecords = streamScanInitRecords2(streamptr);
+  int nrecords = streamScanInitRecords2(streamptr);
 
   for (int rindex = 0; rindex <= nrecords; rindex++)
     {
@@ -351,20 +355,20 @@ extScanTimestep2(stream_t *streamptr)
           break;
         }
 
-      const size_t recsize = (size_t) (fileGetPos(fileID) - recpos);
+      size_t recsize = (size_t) (fileGetPos(fileID) - recpos);
 
       extInqHeader(extp, header);
 
-      const int vdate = header[0];
-      const int vtime = 0;
-      const int rcode = header[1];
-      const int rlevel = header[2];
-      const int param = cdiEncodeParam(rcode, 255, 255);
+      int vdate = header[0];
+      int vtime = 0;
+      int rcode = header[1];
+      int rlevel = header[2];
+      int param = cdiEncodeParam(rcode, 255, 255);
 
       if (rindex == 0)
         {
           taxis->type = TAXIS_ABSOLUTE;
-          taxis->vdatetime = cdiDateTime_set(vdate, vtime);
+          taxis->vDateTime = cdiDateTime_set(vdate, vtime);
         }
 
       bool nextstep = false;
@@ -409,14 +413,10 @@ extScanTimestep2(stream_t *streamptr)
   int nrecs = 0;
   for (int recID = 0; recID < nrecords; recID++)
     {
-      if (!records[recID].used)
-        {
-          vlistDefVarTimetype(vlistID, records[recID].varID, TIME_CONSTANT);
-        }
+      if (records[recID].used)
+        nrecs++;
       else
-        {
-          nrecs++;
-        }
+        vlistDefVarTimetype(vlistID, records[recID].varID, TIME_CONSTANT);
     }
   streamptr->tsteps[tsID].nrecs = nrecs;
 
@@ -434,7 +434,7 @@ extInqContents(stream_t *streamptr)
 
   extScanTimestep1(streamptr);
 
-  const int status = (streamptr->ntsteps == -1) ? extScanTimestep2(streamptr) : 0;
+  int status = (streamptr->ntsteps == -1) ? extScanTimestep2(streamptr) : 0;
 
   fileSetPos(streamptr->fileID, 0, SEEK_SET);
 
@@ -459,7 +459,7 @@ extScanTimestep(stream_t *streamptr)
 
       nrecs = streamScanInitRecords(streamptr, tsID);
 
-      const int fileID = streamptr->fileID;
+      int fileID = streamptr->fileID;
 
       fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
@@ -472,24 +472,24 @@ extScanTimestep(stream_t *streamptr)
               break;
             }
 
-          const size_t recsize = (size_t) (fileGetPos(fileID) - recpos);
+          size_t recsize = (size_t) (fileGetPos(fileID) - recpos);
 
           extInqHeader(extp, header);
 
-          const int vdate = header[0];
-          const int vtime = 0;
-          const int rcode = header[1];
-          const int rlevel = header[2];
-          const int param = cdiEncodeParam(rcode, 255, 255);
+          int vdate = header[0];
+          int vtime = 0;
+          int rcode = header[1];
+          int rlevel = header[2];
+          int param = cdiEncodeParam(rcode, 255, 255);
 
           // if ( rindex == nrecs ) break; gcc-4.5 internal compiler error
           if (rindex == nrecs) continue;
-          const int recID = streamptr->tsteps[tsID].recIDs[rindex];
+          int recID = streamptr->tsteps[tsID].recIDs[rindex];
 
           if (rindex == 0)
             {
               taxis->type = TAXIS_ABSOLUTE;
-              taxis->vdatetime = cdiDateTime_set(vdate, vtime);
+              taxis->vDateTime = cdiDateTime_set(vdate, vtime);
             }
 
           if (param != records[recID].param || rlevel != records[recID].ilevel)
@@ -556,17 +556,17 @@ extReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data, size_
 
   void *extp = streamptr->record->objectp;
 
-  const int vlistID = streamptr->vlistID;
-  const int fileID = streamptr->fileID;
+  int vlistID = streamptr->vlistID;
+  int fileID = streamptr->fileID;
 
-  const double missval = vlistInqVarMissval(vlistID, varID);
-  const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-  const int tsid = streamptr->curTsID;
+  double missval = vlistInqVarMissval(vlistID, varID);
+  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+  int tsid = streamptr->curTsID;
 
-  const off_t currentfilepos = fileGetPos(fileID);
+  off_t currentfilepos = fileGetPos(fileID);
 
-  const int recID = streamptr->vars[varID].recordTable[0].recordID[levID];
-  const off_t recpos = streamptr->tsteps[tsid].records[recID].position;
+  int recID = streamptr->vars[varID].recordTable[0].recordID[levID];
+  off_t recpos = streamptr->tsteps[tsid].records[recID].position;
   fileSetPos(fileID, recpos, SEEK_SET);
   extRead(fileID, extp);
   int header[4];
@@ -586,9 +586,9 @@ extReadVarDP(stream_t *streamptr, int varID, double *data, size_t *nmiss)
 {
   if (CDI_Debug) Message("streamID = %d  varID = %d", streamptr->self, varID);
 
-  const int vlistID = streamptr->vlistID;
-  const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-  const size_t nlevs = (size_t) streamptr->vars[varID].recordTable[0].nlevs;
+  int vlistID = streamptr->vlistID;
+  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+  size_t nlevs = (size_t) streamptr->vars[varID].recordTable[0].nlevs;
 
   for (size_t levID = 0; levID < nlevs; levID++) extReadVarSliceDP(streamptr, varID, (int) levID, &data[levID * gridsize], nmiss);
 }
@@ -598,18 +598,19 @@ extWriteVarSliceDP(stream_t *streamptr, int varID, int levID, const double *data
 {
   if (CDI_Debug) Message("streamID = %d  varID = %d  levID = %d", streamptr->self, varID, levID);
 
-  const int vlistID = streamptr->vlistID;
-  const int fileID = streamptr->fileID;
-  const int tsID = streamptr->curTsID;
+  int vlistID = streamptr->vlistID;
+  int fileID = streamptr->fileID;
+  int tsID = streamptr->curTsID;
+  CdiDateTime vDateTime = streamptr->tsteps[tsID].taxis.vDateTime;
 
   int pdis, pcat, pnum;
   cdiDecodeParam(vlistInqVarParam(vlistID, varID), &pnum, &pcat, &pdis);
 
   int header[4];
-  header[0] = (int) cdiDate_get(streamptr->tsteps[tsID].taxis.vdatetime.date);
+  header[0] = (int) cdiDate_get(vDateTime.date);
   header[1] = pnum;
   header[2] = (int) lround(zaxisInqLevel(vlistInqVarZaxis(vlistID, varID), levID));
-  const int gridID = vlistInqVarGrid(vlistID, varID);
+  int gridID = vlistInqVarGrid(vlistID, varID);
   cdi_check_gridsize_int_limit("EXTRA", gridInqSize(gridID));
   header[3] = (int) gridInqSize(gridID);
 
@@ -626,9 +627,9 @@ extWriteVarDP(stream_t *streamptr, int varID, const double *data)
 {
   if (CDI_Debug) Message("streamID = %d  varID = %d", streamptr->self, varID);
 
-  const int vlistID = streamptr->vlistID;
-  const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-  const size_t nlevs = (size_t) zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
+  int vlistID = streamptr->vlistID;
+  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+  size_t nlevs = (size_t) zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
 
   for (size_t levID = 0; levID < nlevs; levID++) extWriteVarSliceDP(streamptr, varID, (int) levID, &data[levID * gridsize]);
 }
