@@ -1113,7 +1113,7 @@ after_parini(AfterControl &globs, struct Variable *vars)
   else
     {
       fseek(stdin, 0L, SEEK_END);
-      const long length = ftell(stdin);
+      long length = ftell(stdin);
       if (length == 0L) fprintf(stderr, "\n stdin not connected\n");
       fseek(stdin, 0L, SEEK_SET);
     }
