@@ -17,6 +17,7 @@
 #include "cdo_options.h"
 #include "process_int.h"
 #include <mpim_grid.h>
+#include "field_functions.h"
 
 void *
 Arithlat(void *process)
@@ -60,7 +61,7 @@ Arithlat(void *process)
       cdo_taxis_copy_timestep(taxisID2, taxisID1);
       cdo_def_timestep(streamID2, tsID);
 
-      for (int recID = 0; recID < nrecs; recID++)
+      for (int recID = 0; recID < nrecs; ++recID)
         {
           int varID, levelID;
           cdo_inq_record(streamID1, &varID, &levelID);
@@ -69,33 +70,13 @@ Arithlat(void *process)
 
           auto gridID = varList1[varID].gridID;
           const auto gridsize = varList1[varID].gridsize;
+          const auto missval = varList1[varID].missval;
 
           if (gridID != gridID0)
             {
               gridID0 = gridID;
-
-              const auto gridtype = gridInqType(gridID);
-              const auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-              if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_LCC)
-                {
-                  gridID = gridToCurvilinear(gridID, 0);
-                }
-              else if (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED)
-                {
-                  // No conversion necessary
-                }
-              else if (gridtype == GRID_GME)
-                {
-                  gridID = gridToUnstructured(gridID, 0);
-                }
-              else
-                {
-                  if (gridtype == GRID_GAUSSIAN_REDUCED)
-                    cdo_abort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!",
-                              gridNamePtr(gridtype));
-                  else
-                    cdo_abort("Unsupported grid type: %s", gridNamePtr(gridtype));
-                }
+              gridID = generate_full_point_grid(gridID);
+              if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
 
               scale.resize(gridsize);
               gridInqYvals(gridID, scale.data());
@@ -109,10 +90,18 @@ Arithlat(void *process)
                 for (size_t i = 0; i < gridsize; ++i) scale[i] = 1.0 / std::cos(scale[i]);
 
               if (Options::cdoVerbose)
-                for (unsigned i = 0; i < 10; ++i) cdo_print("coslat  %3d  %g", i + 1, scale[i]);
+                for (int i = 0; i < 10; ++i) cdo_print("coslat  %3d  %g", i + 1, scale[i]);
             }
 
-          for (size_t i = 0; i < gridsize; ++i) array[i] *= scale[i];
+          if (nmiss)
+            {
+              for (size_t i = 0; i < gridsize; ++i)
+                if (!DBL_IS_EQUAL(array[i], missval)) array[i] *= scale[i];
+            }
+          else
+            {
+              for (size_t i = 0; i < gridsize; ++i) array[i] *= scale[i];
+            }
 
           cdo_def_record(streamID2, varID, levelID);
           cdo_write_record(streamID2, array.data(), nmiss);
