@@ -12,7 +12,7 @@
 #include <cdi.h>
 
 #include "param_conversion.h"
-#include "cdo_wtime.h"
+#include "cdo_timer.h"
 #include "process_int.h"
 
 void *
@@ -20,33 +20,33 @@ CDItest(void *process)
 {
   cdo_initialize(process);
 
-  const auto dataIsUnchanged = false;
-  // const auto dataIsUnchanged = data_is_unchanged();
+  auto dataIsUnchanged = false;
+  // auto dataIsUnchanged = data_is_unchanged();
 
-  const auto NCOPY = cdo_operator_add("ncopy", 0, 0, nullptr);
+  auto NCOPY = cdo_operator_add("ncopy", 0, 0, nullptr);
   (void) (NCOPY);  // unused
 
-  const auto operatorID = cdo_operator_id();
+  auto operatorID = cdo_operator_id();
   (void) (operatorID);  // unused
 
   //  operator_input_arg("Number of copies");
-  const auto max_copy = (cdo_operator_argc() == 1) ? parameter_to_int(cdo_operator_argv(0)) : 3;
+  auto max_copy = (cdo_operator_argc() == 1) ? parameter_to_int(cdo_operator_argv(0)) : 3;
 
   int n = 0;
   while (true)
     {
-      const auto startTime = cdo_get_wtime();
+      cdo::timer timer;
 
-      const auto streamID1 = cdo_open_read(0);
+      auto streamID1 = cdo_open_read(0);
 
-      const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-      const auto taxisID1 = vlistInqTaxis(vlistID1);
+      auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+      auto taxisID1 = vlistInqTaxis(vlistID1);
 
-      const auto vlistID2 = vlistDuplicate(vlistID1);
-      const auto taxisID2 = taxisDuplicate(taxisID1);
+      auto vlistID2 = vlistDuplicate(vlistID1);
+      auto taxisID2 = taxisDuplicate(taxisID1);
       vlistDefTaxis(vlistID2, taxisID2);
 
-      const auto streamID2 = cdo_open_write(1);
+      auto streamID2 = cdo_open_write(1);
       cdo_def_vlist(streamID2, vlistID2);
 
       Field field;
@@ -57,7 +57,7 @@ CDItest(void *process)
       int tsID = 0;
       while (true)
         {
-          const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+          auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
           if (nrecs == 0) break;
 
           cdo_taxis_copy_timestep(taxisID2, taxisID1);
@@ -90,7 +90,7 @@ CDItest(void *process)
 
       n++;
 
-      cdo_print("Copy number %d: %.2fs", n, cdo_get_wtime() - startTime);
+      cdo_print("Copy number %d: %.2fs", n, timer.elapsed());
 
       if (n == max_copy) break;
     }
