@@ -108,7 +108,7 @@ cmor_check_eval(int vlistID, int nvars, const std::vector<CmorVar> &vars)
 }
 
 void
-cmor_check_prep(CmorVar &var, const long gridsize, const double missval, const double *const array)
+cmor_check_prep(CmorVar &var, long gridsize, double missval, const double *const array)
 {
   if (var.checkvalid || var.check_min_mean_abs || var.check_max_mean_abs)
     {
