@@ -66,7 +66,7 @@ Dayarith(void *process)
   FieldVector2D vars2;
   fields_from_vlist(vlistID2, vars2, FIELD_VEC | FIELD_NAT);
 
-  CdiDate vDate2 = {};
+  CdiDate vDate2{};
   int tsID = 0;
   int tsID2 = 0;
   while (true)
