@@ -21,18 +21,19 @@
 #include "cimdOmp.h"
 #include "pmlist.h"
 #include "param_conversion.h"
+#include "field_functions.h"
 
 static void
 detrend(const long nts, const Varray<double> &deltaTS0, const double missval1, const Varray<double> &array1, Varray<double> &array2)
 {
-  const auto missval2 = missval1;
+  auto missval2 = missval1;
   double sumj = 0.0, sumjj = 0.0;
   double sumx = 0.0, sumjx = 0.0;
   long n = 0;
   for (long j = 0; j < nts; ++j)
-    if (!DBL_IS_EQUAL(array1[j], missval1))
+    if (!dbl_is_equal(array1[j], missval1))
       {
-        const auto zj = deltaTS0[j];
+        auto zj = deltaTS0[j];
         sumj += zj;
         sumjj += zj * zj;
         sumjx += zj * array1[j];
@@ -40,8 +41,8 @@ detrend(const long nts, const Varray<double> &deltaTS0, const double missval1, c
         n++;
       }
 
-  const auto work1 = DIVMN(SUBMN(sumjx, DIVMN(MULMN(sumj, sumx), n)), SUBMN(sumjj, DIVMN(MULMN(sumj, sumj), n)));
-  const auto work2 = SUBMN(DIVMN(sumx, n), MULMN(DIVMN(sumj, n), work1));
+  auto work1 = DIVMN(SUBMN(sumjx, DIVMN(MULMN(sumj, sumx), n)), SUBMN(sumjj, DIVMN(MULMN(sumj, sumj), n)));
+  auto work2 = SUBMN(DIVMN(sumx, n), MULMN(DIVMN(sumj, n), work1));
 
   for (long j = 0; j < nts; ++j) array2[j] = SUBMN(array1[j], ADDMN(work2, MULMN(work1, deltaTS0[j])));
 }
@@ -55,7 +56,7 @@ computeDeltaTS0(bool tstepIsEqual, int nts, int calendar, DateTimeList &dtlist,
 
   for (int tsID = 0; tsID < nts; ++tsID)
     {
-      const auto vDateTime = dtlist.get_vDateTime(tsID);
+      auto vDateTime = dtlist.get_vDateTime(tsID);
       if (tstepIsEqual) check_time_increment(tsID, calendar, vDateTime, checkTimeIncr);
       deltaTS0[tsID] = tstepIsEqual ? (double) tsID : delta_time_step_0(tsID, calendar, vDateTime, julianDate0, deltat1);
     }
@@ -64,13 +65,13 @@ computeDeltaTS0(bool tstepIsEqual, int nts, int calendar, DateTimeList &dtlist,
 static void
 detrendGetParameter(bool &tstepIsEqual)
 {
-  const auto pargc = cdo_operator_argc();
+  auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto pargv = cdo_get_oper_argv();
+      const auto &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
-      kvlist.name = "TREND";
+      kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
@@ -100,28 +101,28 @@ Detrend(void *process)
   auto tstepIsEqual = true;
   detrendGetParameter(tstepIsEqual);
 
-  const auto streamID1 = cdo_open_read(0);
+  auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-  const auto vlistID2 = vlistDuplicate(vlistID1);
+  auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+  auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const auto taxisID1 = vlistInqTaxis(vlistID1);
-  const auto taxisID2 = taxisDuplicate(taxisID1);
+  auto taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const auto streamID2 = cdo_open_write(1);
+  auto streamID2 = cdo_open_write(1);
   cdo_def_vlist(streamID2, vlistID2);
 
   VarList varList;
   varListInit(varList, vlistID1);
 
-  const auto nvars = vlistNvars(vlistID1);
+  auto nvars = vlistNvars(vlistID1);
   FieldVector3D vars;
 
   int tsID = 0;
   while (true)
     {
-      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       constexpr size_t NALLOC_INC = 1024;
@@ -142,22 +143,22 @@ Detrend(void *process)
       tsID++;
     }
 
-  const auto nts = tsID;
+  auto nts = tsID;
   Varray<double> deltaTS0(nts);
   Varray2D<double> array1_2D(Threading::ompNumThreads, Varray<double>(nts));
   Varray2D<double> array2_2D(Threading::ompNumThreads, Varray<double>(nts));
 
-  const auto calendar = taxisInqCalendar(taxisID1);
+  auto calendar = taxisInqCalendar(taxisID1);
   computeDeltaTS0(tstepIsEqual, nts, calendar, dtlist, deltaTS0);
 
   for (varID = 0; varID < nvars; ++varID)
     {
-      auto nsteps = (varList[varID].timetype == TIME_CONSTANT) ? 1 : nts;
-      auto missval = varList[varID].missval;
-      auto fieldMemType = varList[varID].memType;
-      auto gridsize = varList[varID].gridsize;
-      const auto nlevels = varList[varID].nlevels;
-      for (levelID = 0; levelID < nlevels; ++levelID)
+      const auto &var = varList[varID];
+      auto nsteps = var.isConstant ? 1 : nts;
+      auto missval = var.missval;
+      auto fieldMemType = var.memType;
+      auto gridsize = var.gridsize;
+      for (levelID = 0; levelID < var.nlevels; ++levelID)
         {
 #ifdef _OPENMP
 #pragma omp parallel for default(none) schedule(static) \
@@ -165,7 +166,7 @@ Detrend(void *process)
 #endif
           for (size_t i = 0; i < gridsize; ++i)
             {
-              const auto ompthID = cdo_omp_get_thread_num();
+              auto ompthID = cdo_omp_get_thread_num();
               auto &array1 = array1_2D[ompthID];
               auto &array2 = array2_2D[ompthID];
 
@@ -191,9 +192,9 @@ Detrend(void *process)
 
       for (varID = 0; varID < nvars; ++varID)
         {
-          if (tsID && varList[varID].timetype == TIME_CONSTANT) continue;
-          const auto nlevels = varList[varID].nlevels;
-          for (levelID = 0; levelID < nlevels; ++levelID)
+          const auto &var = varList[varID];
+          if (tsID && var.isConstant) continue;
+          for (levelID = 0; levelID < var.nlevels; ++levelID)
             {
               cdo_def_record(streamID2, varID, levelID);
               auto &field = vars[tsID][varID][levelID];
