@@ -33,28 +33,28 @@
 #include "cdo_options.h"
 #include "util_files.h"
 #include "cimdOmp.h"
+#include "field_functions.h"
 
-struct ens_file_t
+struct EnsstatFile
 {
+  VarList varList;
   CdoStreamID streamID;
   int vlistID;
-  size_t nmiss[2];
   double missval[2];
-  Varray<double> array[2];
+  Field field[2];
 };
 
-struct ensstat_arg_t
+struct EnsstatArg
 {
   int t;
   int varID[2];
   int levelID[2];
-  int vlistID1;
   CdoStreamID streamID2;
   int nfiles;
-  ens_file_t *efData;
+  EnsstatFile *efData;
   double *array2Data;
   double *count2Data;
-  Field *fieldsData;
+  FieldVector fields;
   int operfunc;
   double pn;
   bool lpctl;
@@ -67,47 +67,53 @@ ensstat_func(void *ensarg)
 {
   if (Options::CDO_task) cdo_omp_set_num_threads(Threading::ompNumThreads);
 
-  auto arg = (ensstat_arg_t *) ensarg;
+  auto arg = (EnsstatArg *) ensarg;
   auto t = arg->t;
   auto nfiles = arg->nfiles;
   auto ef = arg->efData;
-  auto fields = arg->fieldsData;
+  auto &fields = arg->fields;
   auto array2 = arg->array2Data;
   auto count2 = arg->count2Data;
   auto withCountData = arg->withCountData;
 
   auto hasMissvals = false;
   for (int fileID = 0; fileID < nfiles; ++fileID)
-    if (ef[fileID].nmiss[t] > 0) hasMissvals = true;
+    if (ef[fileID].field[t].nmiss > 0) hasMissvals = true;
 
-  const auto gridID = vlistInqVarGrid(arg->vlistID1, arg->varID[t]);
-  auto gridsize = gridInqSize(gridID);
-  auto missval = vlistInqVarMissval(arg->vlistID1, arg->varID[t]);
+  auto varID = arg->varID[t];
+  auto gridsize = ef[0].varList[varID].gridsize;
+  auto missval = ef[0].varList[varID].missval;
+  auto memType = ef[0].field[t].memType;
 
-  std::atomic<size_t> atomicNumMiss{0};
+  std::atomic<size_t> atomicNumMiss{ 0 };
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
-      const auto ompthID = cdo_omp_get_thread_num();
+      auto ompthID = cdo_omp_get_thread_num();
 
       auto &field = fields[ompthID];
       field.missval = missval;
       field.nmiss = 0;
-      for (int fileID = 0; fileID < nfiles; ++fileID)
-        {
-          field.vec_d[fileID] = ef[fileID].array[t][i];
-          if (hasMissvals && DBL_IS_EQUAL(field.vec_d[fileID], ef[fileID].missval[t]))
-            {
-              field.vec_d[fileID] = missval;
-              field.nmiss++;
-            }
-        }
+      if (memType == MemType::Float)
+        for (int fileID = 0; fileID < nfiles; ++fileID) field.vec_d[fileID] = ef[fileID].field[t].vec_f[i];
+      else
+        for (int fileID = 0; fileID < nfiles; ++fileID) field.vec_d[fileID] = ef[fileID].field[t].vec_d[i];
+
+      if (hasMissvals)
+        for (int fileID = 0; fileID < nfiles; ++fileID)
+          {
+            if (dbl_is_equal(field.vec_d[fileID], ef[fileID].missval[t]))
+              {
+                field.vec_d[fileID] = missval;
+                field.nmiss++;
+              }
+          }
 
       array2[i] = arg->lpctl ? field_pctl(field, arg->pn) : field_function(field, arg->operfunc);
 
-      if (DBL_IS_EQUAL(array2[i], field.missval)) atomicNumMiss++;
+      if (dbl_is_equal(array2[i], field.missval)) atomicNumMiss++;
 
       if (withCountData) count2[i] = nfiles - field.nmiss;
     }
@@ -157,13 +163,13 @@ Ensstat(void *process)
 
   addOperators();
 
-  const auto operatorID = cdo_operator_id();
-  const auto operfunc = cdo_operator_f1(operatorID);
+  auto operatorID = cdo_operator_id();
+  auto operfunc = cdo_operator_f1(operatorID);
 
-  const auto lpctl = (operfunc == FieldFunc_Pctl);
+  auto lpctl = (operfunc == FieldFunc_Pctl);
 
   auto argc = cdo_operator_argc();
-  const auto nargc = argc;
+  auto nargc = argc;
 
   double pn = 0.0;
   if (operfunc == FieldFunc_Pctl)
@@ -188,12 +194,12 @@ Ensstat(void *process)
 
   cdo::set_numfiles(nfiles + 8);
 
-  const auto ofilename = cdo_get_stream_name(nfiles);
+  auto ofilename = cdo_get_stream_name(nfiles);
 
   if (!Options::cdoOverwriteMode && FileUtils::file_exists(ofilename) && !FileUtils::user_file_overwrite(ofilename))
     cdo_abort("Outputfile %s already exists!", ofilename);
 
-  std::vector<ens_file_t> ef(nfiles);
+  std::vector<EnsstatFile> ef(nfiles);
 
   FieldVector fields(Threading::ompNumThreads);
   for (int i = 0; i < Threading::ompNumThreads; ++i) fields[i].resize(nfiles);
@@ -202,75 +208,66 @@ Ensstat(void *process)
     {
       ef[fileID].streamID = cdo_open_read(fileID);
       ef[fileID].vlistID = cdo_stream_inq_vlist(ef[fileID].streamID);
+      varListInit(ef[fileID].varList, ef[fileID].vlistID);
     }
 
   // check that the contents is always the same
   for (int fileID = 1; fileID < nfiles; ++fileID) vlist_compare(ef[0].vlistID, ef[fileID].vlistID, CMP_ALL);
 
-  const auto vlistID1 = ef[0].vlistID;
-  const auto vlistID2 = vlistDuplicate(vlistID1);
-  const auto taxisID1 = vlistInqTaxis(vlistID1);
-  const auto taxisID2 = taxisDuplicate(taxisID1);
+  auto vlistID1 = ef[0].vlistID;
+  auto vlistID2 = vlistDuplicate(vlistID1);
+  auto taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const auto gridsizemax = vlistGridsizeMax(vlistID1);
-
-  for (int fileID = 0; fileID < nfiles; ++fileID)
-    {
-      ef[fileID].array[0].resize(gridsizemax);
-      if (Options::CDO_task) ef[fileID].array[1].resize(gridsizemax);
-    }
-
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   Varray<double> array2(gridsizemax);
 
-  const auto nvars = vlistNvars(vlistID2);
+  auto nvars = vlistNvars(vlistID2);
   Varray<double> count2;
   if (withCountData)
     {
       count2.resize(gridsizemax);
       for (int varID = 0; varID < nvars; ++varID)
         {
-          char name[CDI_MAX_NAME];
-          vlistInqVarName(vlistID2, varID, name);
-          strcat(name, "_count");
-          const auto gridID = vlistInqVarGrid(vlistID2, varID);
-          const auto zaxisID = vlistInqVarZaxis(vlistID2, varID);
-          const auto timetype = vlistInqVarTimetype(vlistID2, varID);
-          const auto cvarID = vlistDefVar(vlistID2, gridID, zaxisID, timetype);
-          cdiDefKeyString(vlistID2, cvarID, CDI_KEY_NAME, name);
+          auto name = ef[0].varList[varID].name + "_count";
+          auto gridID = ef[0].varList[varID].gridID;
+          auto zaxisID = ef[0].varList[varID].zaxisID;
+          auto timetype = ef[0].varList[varID].timetype;
+          auto cvarID = vlistDefVar(vlistID2, gridID, zaxisID, timetype);
+          cdiDefKeyString(vlistID2, cvarID, CDI_KEY_NAME, name.c_str());
           vlistDefVarDatatype(vlistID2, cvarID, CDI_DATATYPE_INT16);
           if (cvarID != (varID + nvars)) cdo_abort("Internal error, varIDs do not match!");
         }
     }
 
-  const auto streamID2 = cdo_open_write(nfiles);
+  auto streamID2 = cdo_open_write(nfiles);
   cdo_def_vlist(streamID2, vlistID2);
 
-  ensstat_arg_t ensstat_arg;
-  ensstat_arg.vlistID1 = vlistID1;
-  ensstat_arg.streamID2 = streamID2;
-  ensstat_arg.nfiles = nfiles;
-  ensstat_arg.array2Data = array2.data();
-  ensstat_arg.count2Data = count2.data();
-  ensstat_arg.fieldsData = fields.data();
-  ensstat_arg.operfunc = operfunc;
-  ensstat_arg.pn = pn;
-  ensstat_arg.lpctl = lpctl;
-  ensstat_arg.withCountData = withCountData;
-  ensstat_arg.nvars = nvars;
-  ensstat_arg.t = 0;
+  EnsstatArg ensstatArg;
+  ensstatArg.streamID2 = streamID2;
+  ensstatArg.nfiles = nfiles;
+  ensstatArg.array2Data = array2.data();
+  ensstatArg.count2Data = count2.data();
+  ensstatArg.fields.resize(Threading::ompNumThreads);
+  for (int i = 0; i < Threading::ompNumThreads; ++i) ensstatArg.fields[i].resize(nfiles);
+  ensstatArg.operfunc = operfunc;
+  ensstatArg.pn = pn;
+  ensstatArg.lpctl = lpctl;
+  ensstatArg.withCountData = withCountData;
+  ensstatArg.nvars = nvars;
+  ensstatArg.t = 0;
 
   auto printWarning = false;
   auto printError = false;
   int t = 0;
   int tsID = 0;
-  do
-    {
+  do {
       nrecs0 = cdo_stream_inq_timestep(ef[0].streamID, tsID);
       for (int fileID = 1; fileID < nfiles; ++fileID)
         {
-          const auto streamID = ef[fileID].streamID;
-          const auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
+          auto streamID = ef[fileID].streamID;
+          auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
           if (nrecs != nrecs0)
             {
               if (nrecs == 0)
@@ -306,29 +303,24 @@ Ensstat(void *process)
           for (int fileID = 0; fileID < nfiles; ++fileID)
             {
               cdo_inq_record(ef[fileID].streamID, &varID, &levelID);
-              ef[fileID].missval[t] = vlistInqVarMissval(ef[fileID].vlistID, varID);
+              ef[fileID].missval[t] = ef[fileID].varList[varID].missval;
+              ef[fileID].field[t].init(ef[fileID].varList[varID]);
             }
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
-          for (int fileID = 0; fileID < nfiles; ++fileID)
-            {
-              cdo_read_record(ef[fileID].streamID, ef[fileID].array[t].data(), &ef[fileID].nmiss[t]);
-            }
+          for (int fileID = 0; fileID < nfiles; ++fileID) { cdo_read_record(ef[fileID].streamID, ef[fileID].field[t]); }
 
-          ensstat_arg.efData = ef.data();
-          ensstat_arg.varID[t] = varID;
-          ensstat_arg.levelID[t] = levelID;
+          ensstatArg.efData = ef.data();
+          ensstatArg.varID[t] = varID;
+          ensstatArg.levelID[t] = levelID;
           if (Options::CDO_task)
             {
-              task->start(ensstat_func, &ensstat_arg);
+              task->start(ensstat_func, &ensstatArg);
               task->wait();
               // t = !t;
             }
-          else
-            {
-              ensstat_func(&ensstat_arg);
-            }
+          else { ensstat_func(&ensstatArg); }
         }
 
       tsID++;
