@@ -32,7 +32,6 @@
 #include "param_conversion.h"
 #include <mpim_grid.h>
 #include "eigen_solution.h"
-#include "timer.h"
 #include "datetime.h"
 #include "eof_mode.h"
 
@@ -53,18 +52,11 @@ EOF3d(void *process)
   bool missval_warning = false;
   size_t nmiss;
   int ngrids;
-  int timer_cov = 0, timer_eig = 0;
 
   int calendar = CALENDAR_STANDARD;
 
   double sum_w;
 
-  if (Options::Timer)
-    {
-      timer_cov = timer_new("Timeof cov");
-      timer_eig = timer_new("Timeof eig");
-    }
-
   cdo_initialize(process);
 
   // clang-format off
@@ -244,8 +236,6 @@ EOF3d(void *process)
 
       npack = 0;  // TODO already set to 0
 
-      if (Options::Timer) timer_start(timer_cov);
-
       for (size_t i = 0; i < temp_size; ++i)
         {
           if (datacounts[varID][i] > 1)
@@ -297,10 +287,6 @@ EOF3d(void *process)
       if (Options::cdoVerbose) cdo_print("calculated cov-matrix");
 
       // SOLVE THE EIGEN PROBLEM
-      if (Options::Timer) timer_stop(timer_cov);
-
-      if (Options::Timer) timer_start(timer_eig);
-
       if (Options::cdoVerbose) cdo_print("Processed correlation matrix for var %d | npack: %zu", varID + 1, n);
 
       if (eigen_mode == JACOBI)
@@ -313,8 +299,6 @@ EOF3d(void *process)
 
       for (size_t eofID = 0; eofID < n; eofID++) eigenvalues[varID][eofID][0] = eigv[eofID];
 
-      if (Options::Timer) timer_stop(timer_eig);
-
       for (int eofID = 0; eofID < n_eig; eofID++)
         {
           double *eigenvec = eigenvectors[varID][eofID].data();
