@@ -421,92 +421,124 @@ add_operators(void)
   // clang-format on
 }
 
+class ModuleGridboxstat
+{
+  int lastgrid = -1;
+  bool wstatus = false;
+
+  int operfunc;
+
+  CdoStreamID streamID1;
+  CdoStreamID streamID2;
+
+  int taxisID1;
+  int taxisID2;
+
+  Field field1, field2;
+  VarList varList1, varList2;
+
+  int xinc;
+  int yinc;
+
+  bool needWeights;
+
+public:
+  void
+  init(void *process)
+  {
+    cdo_initialize(process);
+
+    operator_input_arg("xinc, yinc");
+    operator_check_argc(2);
+    xinc = parameter_to_int(cdo_operator_argv(0));
+    yinc = parameter_to_int(cdo_operator_argv(1));
+
+    add_operators();
+
+    auto operatorID = cdo_operator_id();
+    operfunc = cdo_operator_f1(operatorID);
+    needWeights = (cdo_operator_f2(operatorID) != 0);
+
+    streamID1 = cdo_open_read(0);
+
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
+
+    taxisID1 = vlistInqTaxis(vlistID1);
+    taxisID2 = taxisDuplicate(taxisID1);
+    vlistDefTaxis(vlistID2, taxisID2);
+
+    auto ngrids = vlistNgrids(vlistID1);
+    if (ngrids > 1) cdo_abort("Too many different grids!");
+
+    auto gridID1 = vlistGrid(vlistID1, 0);
+
+    auto gridID2 = genBoxGrid(gridID1, xinc, yinc);
+    for (int index = 0; index < ngrids; ++index) vlistChangeGridIndex(vlistID2, index, gridID2);
+
+    streamID2 = cdo_open_write(1);
+    cdo_def_vlist(streamID2, vlistID2);
+
+    varListInit(varList1, vlistID1);
+    varListInit(varList2, vlistID2);
+
+    auto gridsize1 = gridInqSize(gridID1);
+
+    if (needWeights) field1.weightv.resize(gridsize1);
+  }
+  void
+  run()
+  {
+    int tsID = 0;
+    while (true)
+      {
+        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        if (nrecs == 0) break;
+
+        cdo_taxis_copy_timestep(taxisID2, taxisID1);
+        cdo_def_timestep(streamID2, tsID);
+
+        for (int recID = 0; recID < nrecs; ++recID)
+          {
+            int varID, levelID;
+            cdo_inq_record(streamID1, &varID, &levelID);
+            field1.init(varList1[varID]);
+            cdo_read_record(streamID1, field1);
+
+            if (needWeights && field1.grid != lastgrid)
+              {
+                lastgrid = field1.grid;
+                wstatus = gridcell_weights(field1.grid, field1.weightv);
+              }
+            if (wstatus != 0 && tsID == 0 && levelID == 0)
+              cdo_warning("Grid cell bounds not available, using constant grid cell area weights for variable %s!",
+                          varList1[varID].name);
+
+            field2.init(varList2[varID]);
+            gridboxstat(field1, field2, xinc, yinc, operfunc);
+
+            cdo_def_record(streamID2, varID, levelID);
+            cdo_write_record(streamID2, field2);
+          }
+        tsID++;
+      }
+  }
+  void
+  close()
+  {
+    cdo_stream_close(streamID2);
+    cdo_stream_close(streamID1);
+
+    cdo_finish();
+  }
+};
+
 void *
 Gridboxstat(void *process)
 {
-  int lastgrid = -1;
-  auto wstatus = false;
-
-  cdo_initialize(process);
-
-  operator_input_arg("xinc, yinc");
-  operator_check_argc(2);
-  auto xinc = parameter_to_int(cdo_operator_argv(0));
-  auto yinc = parameter_to_int(cdo_operator_argv(1));
-
-  add_operators();
-
-  auto operatorID = cdo_operator_id();
-  auto operfunc = cdo_operator_f1(operatorID);
-  auto needWeights = (cdo_operator_f2(operatorID) != 0);
-
-  auto streamID1 = cdo_open_read(0);
-
-  auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-  auto vlistID2 = vlistDuplicate(vlistID1);
-
-  auto taxisID1 = vlistInqTaxis(vlistID1);
-  auto taxisID2 = taxisDuplicate(taxisID1);
-  vlistDefTaxis(vlistID2, taxisID2);
-
-  auto ngrids = vlistNgrids(vlistID1);
-  if (ngrids > 1) cdo_abort("Too many different grids!");
-
-  auto gridID1 = vlistGrid(vlistID1, 0);
-
-  auto gridID2 = genBoxGrid(gridID1, xinc, yinc);
-  for (int index = 0; index < ngrids; ++index) vlistChangeGridIndex(vlistID2, index, gridID2);
-
-  auto streamID2 = cdo_open_write(1);
-  cdo_def_vlist(streamID2, vlistID2);
-
-  VarList varList1, varList2;
-  varListInit(varList1, vlistID1);
-  varListInit(varList2, vlistID2);
-
-  auto gridsize1 = gridInqSize(gridID1);
-
-  Field field1, field2;
-  if (needWeights) field1.weightv.resize(gridsize1);
-
-  int tsID = 0;
-  while (true)
-    {
-      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-      if (nrecs == 0) break;
-
-      cdo_taxis_copy_timestep(taxisID2, taxisID1);
-      cdo_def_timestep(streamID2, tsID);
-
-      for (int recID = 0; recID < nrecs; ++recID)
-        {
-          int varID, levelID;
-          cdo_inq_record(streamID1, &varID, &levelID);
-          field1.init(varList1[varID]);
-          cdo_read_record(streamID1, field1);
-
-          if (needWeights && field1.grid != lastgrid)
-            {
-              lastgrid = field1.grid;
-              wstatus = gridcell_weights(field1.grid, field1.weightv);
-            }
-          if (wstatus != 0 && tsID == 0 && levelID == 0)
-            cdo_warning("Grid cell bounds not available, using constant grid cell area weights for variable %s!",
-                        varList1[varID].name);
-
-          field2.init(varList2[varID]);
-          gridboxstat(field1, field2, xinc, yinc, operfunc);
-
-          cdo_def_record(streamID2, varID, levelID);
-          cdo_write_record(streamID2, field2);
-        }
-      tsID++;
-    }
-
-  cdo_stream_close(streamID2);
-  cdo_stream_close(streamID1);
-
-  cdo_finish();
-
+  ModuleGridboxstat gridboxstat;
+  gridboxstat.init(process);
+  gridboxstat.run();
+  gridboxstat.close();
   return nullptr;
 }
