@@ -74,7 +74,7 @@ Input(void *process)
     }
   else { operator_check_argc(0); }
 
-  const int nlevs = (zaxisID == -1) ? 1 : zaxisInqSize(zaxisID);
+  int nlevs = (zaxisID == -1) ? 1 : zaxisInqSize(zaxisID);
 
   double dlevel = 0;
   int nrecs = 0;
