@@ -360,7 +360,7 @@ gengridcell(int gridID1, size_t gridsize2, const std::vector<long> &cellidx)
   auto setDimName = (gridtype == GRID_CURVILINEAR);
   if (gridtype == GRID_CURVILINEAR) gridtype = GRID_UNSTRUCTURED;
 
-  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_GENERIC) return -1;
+  if (gridtype != GRID_UNSTRUCTURED && !is_healpix_grid(gridID1) && gridtype != GRID_GENERIC) return -1;
 
   auto gridID2 = gridCreate(gridtype, gridsize2);
 
@@ -749,16 +749,11 @@ selbox_cell_grid(int gridID1)
   auto gridtype = gridInqType(gridID1);
   auto gridsize1 = gridInqSize(gridID1);
 
-  if (gridtype != GRID_UNSTRUCTURED) cdo_abort("Internal problem, wrong grid type!");
+  if (gridtype != GRID_UNSTRUCTURED && !is_healpix_grid(gridID1)) cdo_abort("Internal problem, wrong grid type!");
 
-  if (!gridHasCoordinates(gridID1))
-    {
-      auto reference = dereferenceGrid(gridID1);
-      if (reference.isValid) gridID1 = reference.gridID;
-      if (reference.notFound) cdo_abort("Reference to source grid not found!");
-    }
-
-  if (!gridHasCoordinates(gridID1)) cdo_abort("Grid cell coordinates missing!");
+  auto gridID0 = gridID1;
+  gridID1 = generate_full_grid(gridID1);
+  if (!gridHasCoordinates(gridID1)) cdo_abort("Cell center coordinates missing!");
 
   {
     Varray<double> xvals(gridsize1), yvals(gridsize1);
@@ -797,6 +792,7 @@ selbox_cell_grid(int gridID1)
   }
 
   selboxInfo.gridID2 = gengridcell(gridID1, gridsize2, cellidx);
+  if (gridID0 != gridID1) gridDestroy(gridID1);
 
   return selboxInfo;
 }
@@ -989,10 +985,10 @@ get_selboxInfo(int vlistID1, int vlistID2, bool operIndexBox)
       //                     || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC)
       //                     || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_STERE);
       if (isReg2dGeoGrid || projHasGeoCoords || (operIndexBox && (gridtype == GRID_GENERIC || gridtype == GRID_PROJECTION))
-          || (!operIndexBox && gridtype == GRID_UNSTRUCTURED))
+          || (!operIndexBox && (gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID1))))
         {
           if (operIndexBox) { selboxInfo.push_back(selbox_index_grid(gridID1)); }
-          else { selboxInfo.push_back((gridtype == GRID_UNSTRUCTURED) ? selbox_cell_grid(gridID1) : selbox_lonlat_grid(gridID1)); }
+          else { selboxInfo.push_back((gridtype == GRID_UNSTRUCTURED || is_healpix_grid(gridID1)) ? selbox_cell_grid(gridID1) : selbox_lonlat_grid(gridID1)); }
 
           vlistChangeGridIndex(vlistID2, index, selboxInfo.back().gridID2);
         }
@@ -1005,7 +1001,7 @@ get_selboxInfo(int vlistID1, int vlistID2, bool operIndexBox)
   if (Options::cdoVerbose)
     {
       for (const auto &sb : selboxInfo)
-        if (sb.gridtype != GRID_UNSTRUCTURED)
+        if (sb.gridtype != GRID_UNSTRUCTURED && !is_healpix_grid(sb.gridID1))
           {
             cdo_print("box1 - idx1,idx2,idy1,idy2: %ld,%ld,%ld,%ld", sb.lon21 + 1, sb.lon22 + 1, sb.lat1 + 1, sb.lat2 + 1);
             cdo_print("box2 - idx1,idx2,idy1,idy2: %ld,%ld,%ld,%ld", sb.lon11 + 1, sb.lon12 + 1, sb.lat1 + 1, sb.lat2 + 1);
@@ -1131,7 +1127,7 @@ public:
 
                 const auto &sb = selboxInfo[get_grid_index(varList1[varID].gridID, selboxInfo)];
 
-                if (operatorID == SELLONLATBOX && sb.gridtype == GRID_UNSTRUCTURED)
+                if (operatorID == SELLONLATBOX && (sb.gridtype == GRID_UNSTRUCTURED || is_healpix_grid(sb.gridID1)))
                   window_cell(field1, field2, sb.cellidx);
                 else
                   window_box(field1, field2, sb.lat1, sb.lat2, sb.lon11, sb.lon12, sb.lon21, sb.lon22);
