@@ -261,7 +261,7 @@ delete_attribute(int cdiID, int varID, const std::string &attName)
       // clang-format off
       if      (attName == "long_name") cdiDeleteKey(cdiID, varID, CDI_KEY_LONGNAME);
       else if (attName == "units")     cdiDeleteKey(cdiID, varID, CDI_KEY_UNITS);
-      // clang-format off
+      // clang-format on
     }
 }
 
@@ -323,8 +323,7 @@ set_attributes(const KVList &kvlist, int vlistID)
         {
           if (kv.nvalues == 0 || (kv.nvalues == 1 && kv.values[0].empty()))
             {
-              for (int idx = 0; idx < numVars; ++idx)
-                delete_attribute(cdiID, varIDs[idx], attName);
+              for (int idx = 0; idx < numVars; ++idx) delete_attribute(cdiID, varIDs[idx], attName);
             }
           else
             {
@@ -333,9 +332,9 @@ set_attributes(const KVList &kvlist, int vlistID)
               if (kv.nvalues == 1)
                 {
                   const auto &value = kv.values[0];
-                  if (value.size() > 2 && value[0] == '{' && value[value.size()-1] == '}')
+                  if (value.size() > 2 && value[0] == '{' && value[value.size() - 1] == '}')
                     {
-                      attrValues = get_attribute(vlistID, value.substr(1,value.size()-2), dtype);
+                      attrValues = get_attribute(vlistID, value.substr(1, value.size() - 2), dtype);
                       useAttrValues = true;
                     }
                 }
@@ -344,105 +343,127 @@ set_attributes(const KVList &kvlist, int vlistID)
 
               if (dtype == -1) dtype = literals_find_datatype(values.size(), values);
 
-              for (int idx = 0; idx < numVars; ++idx)
-                set_attribute(cdiID, varIDs[idx], attName, dtype, values);
+              for (int idx = 0; idx < numVars; ++idx) set_attribute(cdiID, varIDs[idx], attName, dtype, values);
             }
         }
     }
 }
 
+class ModuleSetattribute
+{
+  CdoStreamID streamID1;
+  CdoStreamID streamID2;
+
+  int taxisID1;
+  int taxisID2;
+
+  Field field;
+  bool dataIsUnchanged;
+
+  VarList varList1;
+
+public:
+  void
+  init(void *process)
+  {
+
+    cdo_initialize(process);
+
+    cdo_operator_add("setattribute", 0, 0, "attributes");
+
+    dataIsUnchanged = data_is_unchanged();
+
+    auto operatorID = cdo_operator_id();
+
+    operator_input_arg(cdo_operator_enter(operatorID));
+
+    auto natts = cdo_operator_argc();
+    if (natts == 0) cdo_abort("Parameter missing!");
+
+    PMList pmlist;
+    KVList kvlist;
+    kvlist.name = cdo_module_name();
+    if (kvlist.parse_arguments(natts, cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
+    if (Options::cdoVerbose) kvlist.print();
+
+    auto pkvlist = &kvlist;
+    if (natts == 1)
+      {
+        KeyValues &kv = kvlist.front();
+        if (kv.key == "FILE")
+          {
+            if (Options::cdoVerbose) cdo_print("Reading attributes from: %s", kv.values[0]);
+            auto filename = parameter_to_word(kv.values[0].c_str());
+            auto fp = std::fopen(filename, "r");
+            if (fp == nullptr) cdo_abort("Open failed on: %s\n", filename);
+            pmlist.read_namelist(fp, filename);
+            pkvlist = &pmlist.front();
+            std::fclose(fp);
+            if (Options::cdoVerbose) pkvlist->print();
+          }
+      }
+
+    streamID1 = cdo_open_read(0);
+
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
+
+    set_attributes(*pkvlist, vlistID2);
+
+    taxisID1 = vlistInqTaxis(vlistID1);
+    taxisID2 = taxisDuplicate(taxisID1);
+    vlistDefTaxis(vlistID2, taxisID2);
+
+    streamID2 = cdo_open_write(1);
+    cdo_def_vlist(streamID2, vlistID2);
+
+    varListInit(varList1, vlistID1);
+  }
+  void
+  run()
+  {
+    int tsID = 0;
+    while (true)
+      {
+        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        if (nrecs == 0) break;
+
+        cdo_taxis_copy_timestep(taxisID2, taxisID1);
+        cdo_def_timestep(streamID2, tsID);
+
+        for (int recID = 0; recID < nrecs; ++recID)
+          {
+            int varID, levelID;
+            cdo_inq_record(streamID1, &varID, &levelID);
+            cdo_def_record(streamID2, varID, levelID);
+
+            if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+            else
+              {
+                field.init(varList1[varID]);
+                cdo_read_record(streamID1, field);
+                cdo_write_record(streamID2, field);
+              }
+          }
+
+        tsID++;
+      }
+  }
+  void
+  close()
+  {
+    cdo_stream_close(streamID1);
+    cdo_stream_close(streamID2);
+  }
+};
+
 void *
 Setattribute(void *process)
 {
-  cdo_initialize(process);
-
-  cdo_operator_add("setattribute", 0, 0, "attributes");
-
-  auto dataIsUnchanged = data_is_unchanged();
-
-  auto operatorID = cdo_operator_id();
-
-  operator_input_arg(cdo_operator_enter(operatorID));
-
-  auto natts = cdo_operator_argc();
-  if (natts == 0) cdo_abort("Parameter missing!");
-
-  PMList pmlist;
-  KVList kvlist;
-  kvlist.name = cdo_module_name();
-  if (kvlist.parse_arguments(natts, cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
-  if (Options::cdoVerbose) kvlist.print();
-
-  auto pkvlist = &kvlist;
-  if (natts == 1)
-    {
-      KeyValues &kv = kvlist.front();
-      if (kv.key == "FILE")
-        {
-          if (Options::cdoVerbose) cdo_print("Reading attributes from: %s", kv.values[0]);
-          auto filename = parameter_to_word(kv.values[0].c_str());
-          auto fp = std::fopen(filename, "r");
-          if (fp == nullptr) cdo_abort("Open failed on: %s\n", filename);
-          pmlist.read_namelist(fp, filename);
-          pkvlist = &pmlist.front();
-          std::fclose(fp);
-          if (Options::cdoVerbose) pkvlist->print();
-        }
-    }
-
-  auto streamID1 = cdo_open_read(0);
-
-  auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-  auto vlistID2 = vlistDuplicate(vlistID1);
-
-  set_attributes(*pkvlist, vlistID2);
-
-  auto taxisID1 = vlistInqTaxis(vlistID1);
-  auto taxisID2 = taxisDuplicate(taxisID1);
-  vlistDefTaxis(vlistID2, taxisID2);
-
-  auto streamID2 = cdo_open_write(1);
-  cdo_def_vlist(streamID2, vlistID2);
-
-  Field field;
-
-  VarList varList1;
-  varListInit(varList1, vlistID1);
-
-  int tsID = 0;
-  while (true)
-    {
-      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-      if (nrecs == 0) break;
-
-      cdo_taxis_copy_timestep(taxisID2, taxisID1);
-      cdo_def_timestep(streamID2, tsID);
-
-      for (int recID = 0; recID < nrecs; ++recID)
-        {
-          int varID, levelID;
-          cdo_inq_record(streamID1, &varID, &levelID);
-          cdo_def_record(streamID2, varID, levelID);
-
-          if (dataIsUnchanged)
-            {
-              cdo_copy_record(streamID2, streamID1);
-            }
-          else
-            {
-              field.init(varList1[varID]);
-              cdo_read_record(streamID1, field);
-              cdo_write_record(streamID2, field);
-            }
-        }
-
-      tsID++;
-    }
-
-  cdo_stream_close(streamID1);
-  cdo_stream_close(streamID2);
-
   cdo_finish();
-
+  ModuleSetattribute setattribute;
+  setattribute.init(process);
+  setattribute.run();
+  setattribute.close();
   return nullptr;
 }
