@@ -48,20 +48,20 @@ addOperators(void)
 void *
 Timselstat(void *process)
 {
-  TimeStat timestat_date = TimeStat::MEAN;
+  constexpr auto timestatDate{ TimeStat::MEAN };
 
   cdo_initialize(process);
 
   addOperators();
 
-  const auto operatorID = cdo_operator_id();
-  const auto operfunc = cdo_operator_f1(operatorID);
+  auto operatorID = cdo_operator_id();
+  auto operfunc = cdo_operator_f1(operatorID);
 
-  const auto lrange = (operfunc == FieldFunc_Range);
-  const auto lmean = (operfunc == FieldFunc_Mean || operfunc == FieldFunc_Avg);
-  const auto lstd = (operfunc == FieldFunc_Std || operfunc == FieldFunc_Std1);
-  const auto lvarstd = (lstd || operfunc == FieldFunc_Var || operfunc == FieldFunc_Var1);
-  const auto lvars2 = (lvarstd || lrange);
+  auto lrange = (operfunc == FieldFunc_Range);
+  auto lmean = (operfunc == FieldFunc_Mean || operfunc == FieldFunc_Avg);
+  auto lstd = (operfunc == FieldFunc_Std || operfunc == FieldFunc_Std1);
+  auto lvarstd = (lstd || operfunc == FieldFunc_Var || operfunc == FieldFunc_Var1);
+  auto lvars2 = (lvarstd || lrange);
   const int divisor = (operfunc == FieldFunc_Std1 || operfunc == FieldFunc_Var1);
 
   auto field2_stdvar_func = lstd ? field2_std : field2_var;
@@ -69,31 +69,31 @@ Timselstat(void *process)
 
   operator_input_arg("numSets <noffset <nskip>>");
 
-  const auto nargc = cdo_operator_argc();
-  const auto ndates = parameter_to_int(cdo_operator_argv(0));
-  const auto noffset = (nargc > 1) ? parameter_to_int(cdo_operator_argv(1)) : 0;
-  const auto nskip = (nargc > 2) ? parameter_to_int(cdo_operator_argv(2)) : 0;
+  auto nargc = cdo_operator_argc();
+  auto ndates = parameter_to_int(cdo_operator_argv(0));
+  auto noffset = (nargc > 1) ? parameter_to_int(cdo_operator_argv(1)) : 0;
+  auto nskip = (nargc > 2) ? parameter_to_int(cdo_operator_argv(2)) : 0;
 
   if (Options::cdoVerbose) cdo_print("numSets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
 
-  const auto streamID1 = cdo_open_read(0);
+  auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-  const auto vlistID2 = vlistDuplicate(vlistID1);
+  auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+  auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const auto taxisID1 = vlistInqTaxis(vlistID1);
-  const auto taxisID2 = taxisDuplicate(taxisID1);
+  auto taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const auto streamID2 = cdo_open_write(1);
+  auto streamID2 = cdo_open_write(1);
   cdo_def_vlist(streamID2, vlistID2);
 
-  const auto maxrecs = vlistNrecs(vlistID1);
+  auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
-  dtlist.set_stat(timestat_date);
+  dtlist.set_stat(timestatDate);
   dtlist.set_calendar(taxisInqCalendar(taxisID1));
 
   VarList varList;
@@ -112,7 +112,7 @@ Timselstat(void *process)
   int tsID;
   for (tsID = 0; tsID < noffset; ++tsID)
     {
-      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       for (int recID = 0; recID < nrecs; ++recID)
