@@ -167,13 +167,13 @@ addOperators(void)
 void *
 Timstat(void *argument)
 {
-  const TimeStat timestat_date = TimeStat::MEAN;
+  constexpr auto timestatDate{ TimeStat::MEAN };
   CdiDateTime vDateTime0{};
   CdiDateTime vDateTimeN{};
   CdoStreamID streamID3;
   int vlistID3, taxisID3 = -1;
   bool lvfrac = false;
-  double vfrac = 1;
+  double vfrac = 1.0;
 
   cdo_initialize(argument);
 
@@ -190,7 +190,7 @@ Timstat(void *argument)
   auto lstd = (operfunc == FieldFunc_Std || operfunc == FieldFunc_Std1);
   auto lvarstd = (lstd || operfunc == FieldFunc_Var || operfunc == FieldFunc_Var1);
   auto lvars2 = (lvarstd || lrange || lminidx || lmaxidx);
-  const int divisor = (operfunc == FieldFunc_Std1 || operfunc == FieldFunc_Var1);
+  int divisor = (operfunc == FieldFunc_Std1 || operfunc == FieldFunc_Var1);
 
   auto field2_stdvar_func = lstd ? field2_std : field2_var;
   auto fieldc_stdvar_func = lstd ? fieldc_std : fieldc_var;
@@ -208,7 +208,10 @@ Timstat(void *argument)
       else if (oargc > 1)
         cdo_abort("Too many arguments!");
     }
-  else { operator_check_argc(0); }
+  else
+    {
+      operator_check_argc(0);
+    }
 
   auto streamID1 = cdo_open_read(0);
 
@@ -265,7 +268,7 @@ Timstat(void *argument)
   std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
-  dtlist.set_stat(timestat_date);
+  dtlist.set_stat(timestatDate);
   dtlist.set_calendar(taxisInqCalendar(taxisID1));
 
   VarList varList;
@@ -285,6 +288,9 @@ Timstat(void *argument)
   fields_from_vlist(vlistID1, vars1, FIELD_VEC | VARS_MEMTYPE);
   if (lvars2) fields_from_vlist(vlistID1, vars2, FIELD_VEC);
 
+  auto numVars = varList.size();
+  for (size_t varID = 0; varID < numVars; ++varID) varList[varID].memType = vars1[varID][0].memType;
+
   auto ntsteps1 = vlistNtsteps(vlistID1);
 
   if (!Options::cdoVerbose && ntsteps1 > 1) progress::init();
@@ -399,7 +405,10 @@ Timstat(void *argument)
               else
                 fieldc_stdvar_func(rvars1, vars2[varID][levelID], numSets, divisor);
             }
-          else if (lrange) { field2_sub(rvars1, vars2[varID][levelID]); }
+          else if (lrange)
+            {
+              field2_sub(rvars1, vars2[varID][levelID]);
+            }
         }
 
       if (Options::cdoVerbose) cdo_print("%s  vfrac = %g, numSets = %d", datetime_to_string(vDateTimeN), vfrac, numSets);
