@@ -19,17 +19,18 @@
 #include "datetime.h"
 #include "pmlist.h"
 #include "param_conversion.h"
+#include "field_functions.h"
 
 static void
 trendGetParameter(bool &tstepIsEqual)
 {
-  const auto pargc = cdo_operator_argc();
+  auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      auto pargv = cdo_get_oper_argv();
+      const auto &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
-      kvlist.name = "TREND";
+      kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
@@ -56,34 +57,34 @@ Trend(void *process)
   auto tstepIsEqual = true;
   trendGetParameter(tstepIsEqual);
 
-  const auto streamID1 = cdo_open_read(0);
+  auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-  const auto vlistID2 = vlistDuplicate(vlistID1);
+  auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+  auto vlistID2 = vlistDuplicate(vlistID1);
 
   vlistDefNtsteps(vlistID2, 1);
 
-  const auto taxisID1 = vlistInqTaxis(vlistID1);
-  const auto taxisID2 = taxisDuplicate(taxisID1);
+  auto taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   VarList varList1;
   varListInit(varList1, vlistID1);
 
-  const auto nvars = vlistNvars(vlistID1);
+  auto nvars = vlistNvars(vlistID1);
 
-  const auto maxrecs = vlistNrecs(vlistID1);
+  auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
   for (int varID = 0; varID < nvars; ++varID) vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
 
-  const auto streamID2 = cdo_open_write(1);
-  const auto streamID3 = cdo_open_write(2);
+  auto streamID2 = cdo_open_write(1);
+  auto streamID3 = cdo_open_write(2);
 
   cdo_def_vlist(streamID2, vlistID2);
   cdo_def_vlist(streamID3, vlistID2);
 
-  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
   field1.resize(gridsizemax);
@@ -93,39 +94,34 @@ Trend(void *process)
   FieldVector2D work[nwork];
   for (auto &w : work) fields_from_vlist(vlistID1, w, FIELD_VEC, 0);
 
-  const auto calendar = taxisInqCalendar(taxisID1);
+  auto calendar = taxisInqCalendar(taxisID1);
 
   CheckTimeIncr checkTimeIncr;
   JulianDate julianDate0;
   double deltat1 = 0.0;
-  CdiDateTime vDateTime;
+  CdiDateTime vDateTime{};
   int tsID = 0;
   while (true)
     {
-      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       vDateTime = taxisInqVdatetime(taxisID1);
 
       if (tstepIsEqual) check_time_increment(tsID, calendar, vDateTime, checkTimeIncr);
-      const auto zj = tstepIsEqual ? (double) tsID : delta_time_step_0(tsID, calendar, vDateTime, julianDate0, deltat1);
+      auto zj = tstepIsEqual ? (double) tsID : delta_time_step_0(tsID, calendar, vDateTime, julianDate0, deltat1);
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
           int varID, levelID;
           cdo_inq_record(streamID1, &varID, &levelID);
 
-          if (tsID == 0)
-            {
-              recList[recID].varID = varID;
-              recList[recID].levelID = levelID;
-              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
-            }
+          recList[recID].set(varID, levelID);
 
           cdo_read_record(streamID1, field1.vec_d.data(), &field1.nmiss);
 
-          const auto gridsize = varList1[varID].gridsize;
-          const auto missval = varList1[varID].missval;
+          auto gridsize = varList1[varID].gridsize;
+          auto missval = varList1[varID].missval;
 
           auto &sumj = work[0][varID][levelID].vec_d;
           auto &sumjj = work[1][varID][levelID].vec_d;
@@ -134,7 +130,7 @@ Trend(void *process)
           auto &zn = work[4][varID][levelID].vec_d;
 
           for (size_t i = 0; i < gridsize; ++i)
-            if (!DBL_IS_EQUAL(field1.vec_d[i], missval))
+            if (!dbl_is_equal(field1.vec_d[i], missval))
               {
                 sumj[i] += zj;
                 sumjj[i] += zj * zj;
@@ -153,12 +149,12 @@ Trend(void *process)
 
   for (int recID = 0; recID < maxrecs; ++recID)
     {
-      const auto varID = recList[recID].varID;
-      const auto levelID = recList[recID].levelID;
-      const auto gridsize = varList1[varID].gridsize;
-      const auto missval = varList1[varID].missval;
-      const auto missval1 = missval;
-      const auto missval2 = missval;
+      auto [varID, levelID] = recList[recID].get();
+
+      auto gridsize = varList1[varID].gridsize;
+      auto missval = varList1[varID].missval;
+      auto missval1 = missval;
+      auto missval2 = missval;
       field1.size = gridsize;
       field1.missval = missval;
       field2.size = gridsize;
@@ -172,8 +168,8 @@ Trend(void *process)
 
       for (size_t i = 0; i < gridsize; ++i)
         {
-          const auto temp1 = SUBMN(sumjx[i], DIVMN(MULMN(sumj[i], sumx[i]), zn[i]));
-          const auto temp2 = SUBMN(sumjj[i], DIVMN(MULMN(sumj[i], sumj[i]), zn[i]));
+          auto temp1 = SUBMN(sumjx[i], DIVMN(MULMN(sumj[i], sumx[i]), zn[i]));
+          auto temp2 = SUBMN(sumjj[i], DIVMN(MULMN(sumj[i], sumj[i]), zn[i]));
 
           field2.vec_d[i] = DIVMN(temp1, temp2);
           field1.vec_d[i] = SUBMN(DIVMN(sumx[i], zn[i]), MULMN(DIVMN(sumj[i], zn[i]), field2.vec_d[i]));
