@@ -22,14 +22,15 @@
 #include "process_int.h"
 #include "param_conversion.h"
 #include "percentiles_hist.h"
+#include "field_functions.h"
 
 void *
 Ydaypctl(void *process)
 {
   constexpr int MaxDays = 373;
-  CdiDateTime vDateTimes1[MaxDays] = { };
-  CdiDateTime vDateTimes2[MaxDays] = { };
-  long nsets[MaxDays] = { 0 };
+  CdiDateTime vDateTimes1[MaxDays]{};
+  CdiDateTime vDateTimes2[MaxDays]{};
+  long numSets[MaxDays] = { 0 };
   std::vector<bool> vars1(MaxDays, false);
   HistogramSet hsets[MaxDays];
 
@@ -105,18 +106,22 @@ Ydaypctl(void *process)
           hsets[dayOfYear].create(nvars, ntsteps);
 
           for (int varID = 0; varID < nvars; ++varID)
-            hsets[dayOfYear].createVarLevels(varID, varList1[varID].nlevels, varList1[varID].gridsize);
+            {
+              const auto &var = varList1[varID];
+              hsets[dayOfYear].createVarLevels(varID, var.nlevels, var.gridsize);
+            }
         }
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
           int varID, levelID;
           cdo_inq_record(streamID2, &varID, &levelID);
-          field1.init(varList1[varID]);
+          const auto &var = varList1[varID];
+          field1.init(var);
           cdo_read_record(streamID2, field1);
 
           cdo_inq_record(streamID3, &varID, &levelID);
-          field2.init(varList1[varID]);
+          field2.init(var);
           cdo_read_record(streamID3, field2);
 
           hsets[dayOfYear].defVarLevelBounds(varID, levelID, field1, field2);
@@ -140,41 +145,38 @@ Ydaypctl(void *process)
 
       vDateTimes1[dayOfYear] = vDateTime;
 
-      if (!vars1[dayOfYear]) cdo_abort("No data for day %d in %s and %s", dayOfYear, cdo_get_stream_name(1), cdo_get_stream_name(2));
+      if (!vars1[dayOfYear])
+        cdo_abort("No data for day %d in %s and %s", dayOfYear, cdo_get_stream_name(1), cdo_get_stream_name(2));
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
           int varID, levelID;
           cdo_inq_record(streamID1, &varID, &levelID);
+          const auto &var = varList1[varID];
 
-          if (tsID == 0)
-            {
-              recList[recID].varID = varID;
-              recList[recID].levelID = levelID;
-              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
-            }
+          if (tsID == 0) recList[recID].set(varID, levelID);
 
-          if (tsID == 0 && recList[recID].lconst)
+          if (tsID == 0 && var.isConstant)
             {
-              constFields[recID].init(varList1[varID]);
+              constFields[recID].init(var);
               cdo_read_record(streamID1, constFields[recID]);
             }
           else
             {
-              field1.init(varList1[varID]);
+              field1.init(var);
               cdo_read_record(streamID1, field1);
 
               hsets[dayOfYear].addVarLevelValues(varID, levelID, field1);
             }
         }
 
-      nsets[dayOfYear]++;
+      numSets[dayOfYear]++;
       tsID++;
     }
 
   int otsID = 0;
   for (int dayOfYear = 0; dayOfYear < MaxDays; dayOfYear++)
-    if (nsets[dayOfYear])
+    if (numSets[dayOfYear])
       {
         if (decode_month_and_day(vDateTimes1[dayOfYear].date) != decode_month_and_day(vDateTimes2[dayOfYear].date))
           cdo_abort("Verification dates for the day %d of %s and %s are different!", dayOfYear, cdo_get_stream_name(0),
@@ -185,16 +187,12 @@ Ydaypctl(void *process)
 
         for (int recID = 0; recID < maxrecs; ++recID)
           {
-            if (otsID && recList[recID].lconst) continue;
+            auto [varID, levelID] = recList[recID].get();
+            if (otsID && varList1[varID].isConstant) continue;
 
-            const auto varID = recList[recID].varID;
-            const auto levelID = recList[recID].levelID;
             cdo_def_record(streamID4, varID, levelID);
 
-            if (recList[recID].lconst)
-              {
-                cdo_write_record(streamID4, constFields[recID]);
-              }
+            if (varList1[varID].isConstant) { cdo_write_record(streamID4, constFields[recID]); }
             else
               {
                 field1.init(varList1[varID]);
