@@ -145,7 +145,7 @@ struct Variable
   int *samp;
 };
 
-/* clang-format off */
+// clang-format off
 #define    LOW_CLOUD   34
 #define    MID_CLOUD   35
 #define    HIH_CLOUD   36
@@ -193,14 +193,12 @@ struct Variable
 #define        DPSDX  273
 #define        DPSDY  274
 #define  FRESH_WATER  275
-#define      PS_PROG  276  /* PS for prognostic timestep */
+#define      PS_PROG  276  // PS for prognostic timestep
 #define   HALF_PRESS  277
 #define   FULL_PRESS  278
 #define       THETAH  279
 #define       THETAF  280
-/* clang-format on */
-
-void after_read_vct(const char *vctfile, double **vct, int *nvct);
+// clang-format on
 
 void after_gp2sp(const AfterControl &globs, struct Variable *vars, int ccode);
 void after_GP2FC(double *gp, double *fc, long nlat, long nlon, long nlev, long nfc);
@@ -226,7 +224,7 @@ void after_legini_setup(AfterControl &globs, struct Variable *vars);
 
 template <typename... Args>
 void
-afterAbort(const std::string &format, Args const &... args)
+afterAbort(const std::string &format, Args const &...args)
 {
   extern cdo::Task *afterReadTask;
   extern bool afterReadAsync;
