@@ -109,7 +109,10 @@ AddVector(double *dest, const double *src, size_t len, size_t *nmiss, double mis
       *nmiss = array_num_mv(len, dest, missval);
       if (*nmiss == 0) *nmiss = 1;
     }
-  else { array_add_array(len, dest, src); }
+  else
+    {
+      array_add_array(len, dest, src);
+    }
 }
 
 static void
@@ -884,8 +887,8 @@ Omega(double *omega_in, double *divergence, double *u_wind, double *v_wind, doub
 }
 
 void
-geopot_height_halflevel(double *gheight, const double *const ta_fl, const double *const hus_fl, const double *const p_hl,
-                        const long ngp, const long nlev)
+geopot_height_halflevel(double *gheight, const double *const ta_fl, const double *const hus_fl, const double *const p_hl, long ngp,
+                        long nlev)
 {
   // Computes geopotential height on half level
 
@@ -933,8 +936,8 @@ geopot_height_halflevel(double *gheight, const double *const ta_fl, const double
 }
 
 void
-geopot_height_fulllevel(double *gheight, const double *const ta_fl, const double *const hus_fl, const double *const p_hl,
-                        const long ngp, const long nlev)
+geopot_height_fulllevel(double *gheight, const double *const ta_fl, const double *const hus_fl, const double *const p_hl, long ngp,
+                        long nlev)
 {
   // Computes geopotential height on full level
 
@@ -1642,7 +1645,10 @@ after_processML(AfterControl &globs, struct Variable *vars)
               cdo_warning("log surface pressure (code 152) not found - using surface pressure (code 134)!");
               array_copy(globs.DimGP, vars[PS].hybrid, vars[PS_PROG].hybrid);
             }
-          else { afterAbort("surface pressure not found!"); }
+          else
+            {
+              afterAbort("surface pressure not found!");
+            }
         }
       vars[LNPS].needed = vars[LNPS].selected;
 
@@ -1761,7 +1767,10 @@ after_processML(AfterControl &globs, struct Variable *vars)
           height_to_pressure(globs.p_of_height, globs.LevelRequest, globs.NumLevelRequest);
           pressureLevel = globs.p_of_height;
         }
-      else { pressureLevel = globs.LevelRequest; }
+      else
+        {
+          pressureLevel = globs.LevelRequest;
+        }
 
       gen_vert_index(globs.vertIndex, pressureLevel, vars[FULL_PRESS].hybrid, globs.DimGP, globs.NumLevelRequest, globs.NumLevel);
 
@@ -2624,10 +2633,10 @@ after_EchamDependencies(struct Variable *vars, int ncodes, int type, int source)
 void
 after_legini_setup(AfterControl &globs, struct Variable *vars)
 {
-  const long ntr = globs.Truncation;
-  const long nlat = globs.Latitudes;
-  const long dimsp = (ntr + 1) * (ntr + 2);
-  const long pdim = (dimsp / 2) * nlat;
+  long ntr = globs.Truncation;
+  long nlat = globs.Latitudes;
+  long dimsp = (ntr + 1) * (ntr + 2);
+  long pdim = (dimsp / 2) * nlat;
 
   globs.poli = (double *) Malloc(pdim * sizeof(double));
 
