@@ -10,6 +10,7 @@
 #include "cdo_getopt.h"
 #include "util_string.h"
 #include <cstdio>
+#include <sstream>
 #include <cstdlib>
 
 #include <vector>
@@ -19,14 +20,17 @@
 #define CLIOP_DBG false
 
 #include <sys/ioctl.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <unistd.h>
 
-std::string
-CLIOptions::pad_size_terminal(char padWith, const std::string &sourround)
+int
+get_width()
 {
-  size_t terminal_width = 120;
-  if (stderr_is_tty)
+  int terminal_width = 120;
+  struct stat statbuf;
+  fstat(2, &statbuf);
+  if (S_ISCHR(statbuf.st_mode))
     {
 #if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
       CONSOLE_SCREEN_BUFFER_INFO csbi;
@@ -36,12 +40,8 @@ CLIOptions::pad_size_terminal(char padWith, const std::string &sourround)
       columns = csbi.srWindow.Right - csbi.srWindow.Left + 1;
 #elif __APPLE__
 #include <TargetConditionals.h>
-#if TARGET_IPHONE_SIMULATOR
-      // iOS, tvOS, or watchOS Simulator
-#elif TARGET_OS_MACCATALYST
+#if TARGET_OS_MACCATALYST
       // Mac's Catalyst (ports iOS API into Mac, like UIKit).
-#elif TARGET_OS_IPHONE
-      // iOS, tvOS, or watchOS device
 #elif TARGET_OS_MAC
       // Other kinds of Apple platforms
 #else
@@ -49,18 +49,27 @@ CLIOptions::pad_size_terminal(char padWith, const std::string &sourround)
 #endif
 #elif __linux__ || __unix__ || defined(_POSIX_VERSION)
       struct winsize w;
-      ioctl(STDOUT_FILENO, TIOCGWINSZ, &w);
+      ioctl(STDERR_FILENO, TIOCGWINSZ, &w);
       terminal_width = w.ws_col;
 #endif
     }
+  return terminal_width;
+}
+
+std::string
+CLIOptions::pad_size_terminal(char padWith, const std::string &sourround)
+{
+  int terminal_width = get_width();
 
   std::string line;
   if (sourround.empty()) { line = std::string(terminal_width, padWith); }
   else
     {
-      size_t front_pad = 3;
-      size_t num_spaces = 2;
-      size_t space_left = terminal_width - front_pad - num_spaces - sourround.size();
+      int front_pad = 3;
+      int num_spaces = 2;
+      int space_left = terminal_width - front_pad - num_spaces - sourround.size();
+      if (space_left > terminal_width) { space_left = 1; }
+      if (space_left < 0) { space_left = 0; }
 
       line = std::string(front_pad, padWith);
       line += " " + sourround + " ";
@@ -69,38 +78,6 @@ CLIOptions::pad_size_terminal(char padWith, const std::string &sourround)
   return line + "\n";
 }
 
-void
-CLIOptions::usage()
-{
-  std::cerr << pad_size_terminal('-');
-  fprintf(stderr, "  Usage : cdo  [Options]  Operator1  [-Operator2  [-OperatorN]]\n");
-  std::cerr << pad_size_terminal('-');
-
-  fprintf(stderr, "\n");
-  std::cerr << pad_size_terminal('=', "Options");
-
-  set_text_color(stderr, BLUE);
-  print_options_help();
-  reset_text_color(stderr);
-  std::cerr << pad_size_terminal('-');
-
-  std::cerr << pad_size_terminal('=', "Environment Variables");
-  set_text_color(stderr, BLUE);
-  print_envvar_help();
-  reset_text_color(stderr);
-  std::cerr << pad_size_terminal('-');
-
-  fprintf(stderr, "\n");
-
-  fprintf(stderr, "  Operators:\n");
-  fprintf(stderr, "    Use option --operators for a list of all operators.\n");
-
-  fprintf(stderr, "\n");
-  fprintf(stderr, "  CDO version %s, Copyright (C) 2002-2023 MPI für Meteorologie\n", VERSION);
-  fprintf(stderr, "  This is free software and comes with ABSOLUTELY NO WARRANTY\n");
-  fprintf(stderr, "  Report bugs to <https://mpimet.mpg.de/cdo>\n");
-}
-
 std::vector<std::shared_ptr<cdo_option_2>> CLIOptions::optionRegistry;
 std::map<std::string, std::shared_ptr<cdo_option_2>> CLIOptions::optionMap;
 
@@ -113,14 +90,6 @@ bool CLIOptions::print_settings = false;
 bool CLIOptions::print_envvars = false;
 const int CLIOptions::padding = 46;
 
-bool CLIOptions::stderr_is_tty = true;
-
-void
-CLIOptions::set_tty_status(bool tty_status)
-{
-  stderr_is_tty = tty_status;
-}
-
 int
 CLIOptions::parse(std::vector<std::string> p_argv)
 {
@@ -131,22 +100,46 @@ CLIOptions::parse(std::vector<std::string> p_argv)
       const std::map<std::string, std::shared_ptr<cdo_option_2>>::iterator &it = optionMap.find(p_argv[i]);
       if (it == optionMap.end())
         {
+          std::string arg = p_argv[i];
+          bool isLongFrom = arg.size() >= 2 && arg[0] == '-' && arg[1] == '-';
+          bool isShortForm = arg.size() == 2 && arg[0] == '-';
+          if (isLongFrom || isShortForm) { cdo_abort("Option %s not found", p_argv[i]); }
           retval = i;
           break;
         }
       if (it->second->hasArgument)
         {
+          auto &argument = it->second->argument;
           i++;
-          if (i >= p_argv.size() || ((p_argv[i][0] == '-') && !std::get<0>(tokenize_comma_seperated_int_list(p_argv[i]))))
+
+          bool out_of_bounds = i >= p_argv.size();
+
+          auto is_new_option = [&]() {
+            /*
+             * extracted into labmda for a balance of readability and preventing the seqfault that is caused
+             * by checking for this when we are out of bounds. By delaying execution we prevent the seqfault
+             */
+            bool is_not_negative_number = !std::get<0>(tokenize_comma_seperated_int_list(p_argv[i]));
+            bool has_minus_in_first_pos = (p_argv[i][0] == '-');
+            return has_minus_in_first_pos && is_not_negative_number;
+          };
+
+          if (out_of_bounds || is_new_option())
             {
-              if (it->second->default_value.size() > 0) { it->second->argument = it->second->default_value; }
-              else if (!it->second->argumentIsOptional)
-                cdo_abort("missing argument for %s", p_argv[i - 1]);
+              if (argument.default_value.size() > 0) { argument.value = argument.default_value; }
+              else if (!argument.is_optional)
+                {
+                  argument.on_empty_argument(p_argv[i - 1]);
+                  retval = EXIT_REQUESTED;
+                  break;
+                }
             }
-          else { it->second->argument = p_argv[i]; }
+          else { argument.value = p_argv[i]; }
         }
+
       Debug(CLIOP_DBG, "executing option %s", (*it).first);
       it->second->execute();
+
       if (it->second->abortOnExecute)
         {
           retval = EXIT_REQUESTED;
@@ -176,7 +169,7 @@ CLIOptions::get_env_vars()
           if (!setting_ptr->hasArgument || (setting_ptr->hasArgument && *envVarValue))
             {
               Debug(CLIOP_DBG, "Executing envvar %s", setting_ptr->name);
-              setting_ptr->argument = envVarValue ? std::string(envVarValue) : std::string();
+              setting_ptr->argument.value = envVarValue ? std::string(envVarValue) : std::string();
               setting_ptr->execute();
             }
         }
@@ -188,7 +181,7 @@ CLIOptions::print_registry(const std::vector<std::shared_ptr<cdo_option_2>> &p_r
 {
   for (auto &it : p_registry)
     {
-      if (it->argument.size() > 0) fprintf(stderr, "%s = %s\n", it->name.c_str(), it->argument.c_str());
+      if (it->argument.value.size() > 0) fprintf(stderr, "%s = %s\n", it->name.c_str(), it->argument.value.c_str());
     }
 }
 
@@ -230,36 +223,39 @@ CLIOptions::option(const std::string &p_name, const std::string &p_shortForm)
     }
   return optionRegistry.back();
 }
-void
+std::string
 CLIOptions::print_envvar_help()
 {
+  std::stringstream helpString;
   for (auto &it : envvarMap)
     {
       if (!it.second->isInternal)
         {
-          std::string helpString = std::string(4, ' ');
-          helpString += it.second->name;
-          if (it.second->hasArgument) helpString += " <" + it.second->argumentDescription + "> ";
-          int spaceLeft = padding - helpString.size();
-          for (auto &line : it.second->description) { helpString += std::string(spaceLeft, ' ') + line + "\n"; }
-          std::cerr << helpString;
+          helpString << std::string(4, ' ');
+          helpString << it.second->name;
+          if (it.second->hasArgument) helpString << " <" + it.second->argument.description + "> ";
+          int spaceLeft = padding - helpString.str().size();
+          if (spaceLeft < 0) spaceLeft = 0;
+          for (auto &line : it.second->description) { helpString << std::string(spaceLeft, ' ') + line + "\n"; }
         }
     }
+  return helpString.str();
 }
-void
-CLIOptions::print_options_help()
+std::string
+CLIOptions::print_options_help(const std::string p_category)
 {
+  std::stringstream help;
   for (auto &iter : optionMap)
     {
-      if (iter.first.size() != 2 && !iter.second->isInternal)
+      if (iter.first.size() != 2 && !iter.second->isInternal && iter.second->category == p_category)
         {
           auto option = iter.second;
           std::string helpString = "    ";
           if (!option->shortName.empty()) { helpString += option->shortName + ", "; }
           else { helpString += "    "; }
           helpString += option->name + " ";
-          if (option->hasArgument) helpString += " <" + option->argumentDescription + "> ";
-          if (option->hasArgument && option->argumentDescription.empty())
+          if (option->hasArgument) helpString += " <" + option->argument.description + "> ";
+          if (option->hasArgument && option->argument.description.empty())
             {
               std::cerr << "error: help argument of " << option->name << " has no description!" << std::endl;
               exit(0);
@@ -276,13 +272,14 @@ CLIOptions::print_options_help()
               spaceLeft = padding;
             }
           if (option->description.empty()) helpString += "\n";
-          std::cerr << helpString;
+          help << helpString;
         }
     }
+  return help.str();
 }
 #include <algorithm>
 
-void
+cdo_option_2 *
 CLIOptions::option_from_envvar(const std::string &p_envvarName)
 {
   if (envvarMap.find(p_envvarName) == envvarMap.end()) { cdo_abort("Error envvar %s does not exist", p_envvarName); }
@@ -303,22 +300,23 @@ CLIOptions::option_from_envvar(const std::string &p_envvarName)
   newOption->name = optionName;
   newOption->hasArgument = envVar->hasArgument;
   newOption->description = envVar->description;
-  newOption->argumentDescription = envVar->argumentDescription;
+  newOption->argument.description = envVar->argument.description;
   newOption->description = { "This option is generated from " + p_envvarName + " and will overwrite it.",
                              "See help of corresponding environment variable." };
   newOption->effect = envVar->effect;
   newOption->isInternal = envVar->isInternal;
-  newOption->default_value = envVar->default_value;
+  newOption->argument.default_value = envVar->argument.default_value;
   optionMap[optionName] = newOption;
+  return newOption.get();
 }
 
 void
 CLIOptions::print_available_options()
 {
-  for (auto &iter : optionMap) { std::cout << iter.first << std::endl; }
-  std::cout << "_---------------------------------_" << std::endl;
+  for (auto &iter : optionMap) { std::cerr << iter.first << std::endl; }
+  std::cerr << "_---------------------------------_" << std::endl;
   for (auto &iter : optionMap)
     {
-      if (iter.first.size() == 2) std::cout << iter.second->shortName[1] << (iter.second->hasArgument ? ":" : "");
+      if (iter.first.size() == 2) std::cerr << iter.second->shortName[1] << (iter.second->hasArgument ? ":" : "");
     }
 }
