@@ -9,7 +9,7 @@ InternalCdoSyntaxError::InternalCdoSyntaxError(std::vector<std::string>::const_i
 {
 }
 
-CdoSyntaxError::CdoSyntaxError(InternalCdoSyntaxError &e, std::vector<std::string> &p_argv, std::string context)
+CdoSyntaxError::CdoSyntaxError(InternalCdoSyntaxError &e, std::vector<std::string> &p_argv, const std::string &context)
 
     : InternalCdoSyntaxError(e.iter, e.message, e.file, e.line)
 {
