@@ -19,6 +19,7 @@
 #include "cdo_output.h"
 #include "cdo_default_values.h"
 #include "cdo_history.h"
+#include "cdo_query.h"
 #include "process.h"
 
 #include "timer.h"
@@ -38,9 +39,23 @@ FileStream::FileStream(const std::string &p_fileName) : m_filename(p_fileName)
 int
 FileStream::open_read()
 {
+  int fileID = -1;
+
   open_lock();
 
-  const auto fileID = streamOpenRead(m_filename.c_str());
+  if (m_filename.size() > 6 && m_filename.rfind("query:", 0) == 0)
+    {
+      CdiQuery *query = cdiQueryCreate();
+      auto path = set_query_parameter(m_filename.substr(6), query);
+      if (Options::cdoVerbose) cdiQueryPrint(query);
+
+      fileID = streamOpenReadQuery(path.c_str(), query);
+    }
+  else
+    {
+      fileID = streamOpenRead(m_filename.c_str());
+    }
+
   if (fileID < 0) cdi_open_error(fileID, "Open failed on >%s<", m_filename.c_str());
   isopen = true;
 
@@ -66,14 +81,14 @@ FileStream::open_write(int p_filetype)
   if (Options::cdoInteractive)
     {
       struct stat stbuf;
-      const auto rstatus = stat(m_name.c_str(), &stbuf);
+      auto rstatus = stat(m_name.c_str(), &stbuf);
       // If permanent file already exists, query user whether to overwrite or exit
       if (rstatus != -1) query_user_exit(m_name.c_str());
     }
   if (p_filetype == CDI_UNDEFID) p_filetype = CDI_FILETYPE_GRB;
 
 #ifndef HAVE_NC4HDF5_THREADSAFE
-  const auto outputFileTypeIsNetCDF4 = (p_filetype == CDI_FILETYPE_NC4 || p_filetype == CDI_FILETYPE_NC4C);
+  auto outputFileTypeIsNetCDF4 = (p_filetype == CDI_FILETYPE_NC4 || p_filetype == CDI_FILETYPE_NC4C);
   if (inputFileTypeIsNetCDF4 && outputFileTypeIsNetCDF4 && get_process_num() > 1 && Threading::cdoLockIO == false)
     {
       cdo_warning("Using a non-thread-safe NetCDF4/HDF5 library in a multi-threaded environment may lead to erroneous results!");
@@ -84,7 +99,7 @@ FileStream::open_write(int p_filetype)
   // TODO FIX THIS: if (FileStream::timersEnabled()) timer_start(timer_write);
 
   open_lock();
-  const auto fileID = streamOpenWrite(m_filename.c_str(), p_filetype);
+  auto fileID = streamOpenWrite(m_filename.c_str(), p_filetype);
   open_unlock();
 
   // TODO FIX THIS: if(FileStream::timersEnabled()) timer_stop(timer_write);
@@ -107,7 +122,7 @@ FileStream::open_append()
   if (FileStream::timersEnabled()) timer_start(timer_write);
 
   open_lock();
-  const auto fileID = streamOpenAppend(m_filename.c_str());
+  auto fileID = streamOpenAppend(m_filename.c_str());
   open_unlock();
 
   if (FileStream::timersEnabled()) timer_stop(timer_write);
@@ -125,11 +140,11 @@ FileStream::open_append()
 }
 
 void
-FileStream::def_vlist(const int p_vlistID)
+FileStream::def_vlist(int p_vlistID)
 {
   if (m_filetype == CDI_FILETYPE_NCZARR)
     {
-      const auto maxSteps = vlistNtsteps(p_vlistID);
+      auto maxSteps = vlistNtsteps(p_vlistID);
       if (maxSteps >= 0)
         streamDefMaxSteps(m_fileID, maxSteps);
       else
@@ -140,39 +155,36 @@ FileStream::def_vlist(const int p_vlistID)
 
   if (CdoDefault::DataType != CDI_UNDEFID)
     {
-      const auto nvars = vlistNvars(p_vlistID);
+      auto nvars = vlistNvars(p_vlistID);
       for (int varID = 0; varID < nvars; ++varID) vlistDefVarDatatype(p_vlistID, varID, CdoDefault::DataType);
 
       if (CdoDefault::DataType == CDI_DATATYPE_FLT64 || CdoDefault::DataType == CDI_DATATYPE_FLT32)
         {
-          for (int varID = 0; varID < nvars; ++varID)
-            {
-              vlistDefVarAddoffset(p_vlistID, varID, 0.0);
-              vlistDefVarScalefactor(p_vlistID, varID, 1.0);
-            }
+          for (int varID = 0; varID < nvars; ++varID) cdiDeleteKey(p_vlistID, varID, CDI_KEY_ADDOFFSET);
+          for (int varID = 0; varID < nvars; ++varID) cdiDeleteKey(p_vlistID, varID, CDI_KEY_SCALEFACTOR);
         }
     }
 
   if (Options::nsb > 0)
     {
-      const auto nvars = vlistNvars(p_vlistID);
+      auto nvars = vlistNvars(p_vlistID);
       for (int varID = 0; varID < nvars; ++varID) vlistDefVarNSB(p_vlistID, varID, Options::nsb);
     }
 
   if (Options::cdoChunkType != CDI_UNDEFID)
     {
-      const auto nvars = vlistNvars(p_vlistID);
+      auto nvars = vlistNvars(p_vlistID);
       for (int varID = 0; varID < nvars; ++varID) cdiDefKeyInt(p_vlistID, varID, CDI_KEY_CHUNKTYPE, Options::cdoChunkType);
-      const auto ngrids = vlistNgrids(p_vlistID);
+      auto ngrids = vlistNgrids(p_vlistID);
       for (int index = 0; index < ngrids; ++index)
         cdiDefKeyInt(vlistGrid(p_vlistID, index), CDI_GLOBAL, CDI_KEY_CHUNKTYPE, Options::cdoChunkType);
     }
 
   if (Options::cdoChunkSize != CDI_UNDEFID)
     {
-      const auto nvars = vlistNvars(p_vlistID);
+      auto nvars = vlistNvars(p_vlistID);
       for (int varID = 0; varID < nvars; ++varID) cdiDefKeyInt(p_vlistID, varID, CDI_KEY_CHUNKSIZE, Options::cdoChunkSize);
-      const auto ngrids = vlistNgrids(p_vlistID);
+      auto ngrids = vlistNgrids(p_vlistID);
       for (int index = 0; index < ngrids; ++index)
         cdiDefKeyInt(vlistGrid(p_vlistID, index), CDI_GLOBAL, CDI_KEY_CHUNKSIZE, Options::cdoChunkSize);
     }
@@ -200,11 +212,11 @@ int
 FileStream::inq_vlist()
 {
   if (FileStream::timersEnabled()) timer_start(timer_read);
-  const auto vlistID = stream_inq_vlist_locked(m_fileID);
+  auto vlistID = stream_inq_vlist_locked(m_fileID);
   if (FileStream::timersEnabled()) timer_stop(timer_read);
   if (vlistID == -1) cdo_abort("Couldn't read data from input fileID %d!", m_fileID);
 
-  const auto nsubtypes = vlistNsubtypes(vlistID);
+  auto nsubtypes = vlistNsubtypes(vlistID);
   if (nsubtypes > 1) cdo_warning("Subtypes are unsupported, the processing results are possibly wrong!");
 
   if (CdoDefault::TaxisType != CDI_UNDEFID) taxisDefType(vlistInqTaxis(vlistID), CdoDefault::TaxisType);
@@ -223,7 +235,7 @@ FileStream::inq_record(int *const varID, int *const levelID)
 }
 
 void
-FileStream::defRecord(const int varID, const int levelID)
+FileStream::defRecord(int varID, int levelID)
 {
   if (FileStream::timersEnabled()) timer_start(timer_write);
   stream_def_rec_locked(m_fileID, varID, levelID);
@@ -254,13 +266,13 @@ FileStream::read_record(Field *const p_field, size_t *const nmiss)
 }
 
 void
-FileStream::write_record(float *const p_data, const size_t p_nmiss)
+FileStream::write_record(float *const p_data, size_t p_nmiss)
 {
   if (FileStream::timersEnabled()) timer_start(timer_write);
 
-  const auto varID = m_varID;
+  auto varID = m_varID;
   if (varID < (int) m_datarangelist.size())
-    if (m_datarangelist[varID].check_datarange) m_datarangelist[varID].checkDatarange(p_data, p_nmiss);
+    if (m_datarangelist[varID].checkDatarange) m_datarangelist[varID].check_datarange(p_data, p_nmiss);
 
   stream_write_record_float_locked(m_fileID, p_data, p_nmiss);
 
@@ -268,13 +280,13 @@ FileStream::write_record(float *const p_data, const size_t p_nmiss)
 }
 
 void
-FileStream::write_record(double *const p_data, const size_t p_nmiss)
+FileStream::write_record(double *const p_data, size_t p_nmiss)
 {
   if (FileStream::timersEnabled()) timer_start(timer_write);
 
-  const auto varID = m_varID;
+  auto varID = m_varID;
   if (varID < (int) m_datarangelist.size())
-    if (m_datarangelist[varID].check_datarange) m_datarangelist[varID].checkDatarange(p_data, p_nmiss);
+    if (m_datarangelist[varID].checkDatarange) m_datarangelist[varID].check_datarange(p_data, p_nmiss);
 
   stream_write_record_double_locked(m_fileID, p_data, p_nmiss);
 
@@ -282,7 +294,7 @@ FileStream::write_record(double *const p_data, const size_t p_nmiss)
 }
 
 void
-FileStream::write_record(Field *const p_field, const size_t p_nmiss)
+FileStream::write_record(Field *const p_field, size_t p_nmiss)
 {
   write_record(p_field->vec_d.data(), p_nmiss);
 }
@@ -303,10 +315,10 @@ FileStream::copyRecord(CdoStreamID p_destination)
  * -- last Documentation update(2019-06-14) --
  */
 int
-FileStream::inq_timestep(const int p_tsID)
+FileStream::inq_timestep(int p_tsID)
 {
   if (FileStream::timersEnabled()) timer_start(timer_read);
-  const auto nrecs = stream_inq_time_step_locked(m_fileID, p_tsID);
+  auto nrecs = stream_inq_time_step_locked(m_fileID, p_tsID);
   if (FileStream::timersEnabled()) timer_stop(timer_read);
 
   if (p_tsID == 0 && CdoDefault::TaxisType != CDI_UNDEFID) taxisDefType(vlistInqTaxis(m_vlistID), CdoDefault::TaxisType);
@@ -318,7 +330,7 @@ FileStream::inq_timestep(const int p_tsID)
 }
 
 void
-FileStream::def_timestep(const int p_tsID)
+FileStream::def_timestep(int p_tsID)
 {
   if (FileStream::timersEnabled()) timer_start(timer_write);
   // don't use sync -> very slow on GPFS
