@@ -1,35 +1,28 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 #include <cdi.h>
 
 #include "process_int.h"
-#include <mpim_grid.h>
+#include "mpim_grid.h"
+#include "grid_healpix.h"
+#include "gaussian_latitudes.h"
 #include "griddes.h"
 #include "util_string.h"
+#include "dcw_reader.h"
 
-size_t genIcosphereCoords(int subdivisions, bool lbounds, std::vector<double> &xvals, std::vector<double> &yvals,
+size_t genIcosphereCoords(int subdivisions, bool withBounds, std::vector<double> &xvals, std::vector<double> &yvals,
                           std::vector<double> &xbounds, std::vector<double> &ybounds);
 
 static void
 gen_grid_icosphere(GridDesciption &grid, const char *pline)
 {
   int gridtype = GRID_UNSTRUCTURED;
-  bool lbounds = true;
+  bool withBounds = true;
   long b = 0;
 
   if (*pline != 0)
@@ -43,7 +36,7 @@ gen_grid_icosphere(GridDesciption &grid, const char *pline)
       if (!isdigit((int) *pline)) return;
 
       char *endptr = (char *) pline;
-      const long r = strtol(pline, &endptr, 10);
+      auto r = strtol(pline, &endptr, 10);
       if (*endptr == 0 || r != 2) return;
       pline = endptr;
 
@@ -66,7 +59,7 @@ gen_grid_icosphere(GridDesciption &grid, const char *pline)
           if (*pline == 0) return;
           if (*pline == '0')
             {
-              lbounds = false;
+              withBounds = false;
               pline++;
             }
           if (*pline != 0) return;
@@ -74,35 +67,97 @@ gen_grid_icosphere(GridDesciption &grid, const char *pline)
     }
 
   grid.type = gridtype;
-  if (lbounds) grid.nvertex = 3;
+  if (withBounds) grid.nvertex = 3;
 
-  const auto ncells = genIcosphereCoords(b + 1, lbounds, grid.xvals, grid.yvals, grid.xbounds, grid.ybounds);
+  auto ncells = genIcosphereCoords(b + 1, withBounds, grid.xvals, grid.yvals, grid.xbounds, grid.ybounds);
   grid.xsize = ncells;
   grid.ysize = ncells;
-  strcpy(grid.xname, "clon");
-  strcpy(grid.yname, "clat");
-  strcpy(grid.xunits, "radian");
-  strcpy(grid.yunits, "radian");
+  grid.xname = "clon";
+  grid.yname = "clat";
+  grid.xunits = "radian";
+  grid.yunits = "radian";
+}
+
+static void
+gen_grid_zonal(GridDesciption &grid, const char *pline, double inc, double lon1, double lon2, double lat1, double lat2)
+{
+  int gridtype = GRID_LONLAT;
+  bool withBounds = true;
+
+  if (*pline != 0)
+    {
+      if (*pline == '_')
+        pline++;
+      else
+        return;
+
+      if (*pline == 0) return;
+
+      if (!isdigit((int) *pline) && !ispunct((int) *pline)) return;
+
+      char *endptr = (char *) pline;
+      inc = strtod(pline, &endptr);
+      if (*endptr != 0)
+        {
+          pline = endptr;
+          if (*pline == '_')
+            pline++;
+          else
+            return;
+
+          if (*pline == 0) return;
+          if (*pline != 0) return;
+        }
+
+      if (inc < 1.0e-9) inc = 1.0;
+      if (inc > 180.0) cdo_abort("Zonal latitude increment out if range (max=180)!");
+    }
+
+  grid.type = gridtype;
+
+  if (lon1 >= lon2 || lat1 >= lat2) cdo_abort("Invalid grid box: lon1=%g lon2=%g lat1=%g lat2=%g", lon1, lon2, lat1, lat2);
+
+  auto nlon = 1;
+  auto nlat = (size_t) ((lat2 - lat1) / inc + 0.5);
+
+  grid.xvals.resize(nlon, 0.0);
+  grid.yvals.resize(nlat);
+
+  for (size_t i = 0; i < nlat; ++i) grid.yvals[i] = lat1 + inc / 2.0 + i * inc;
+
+  grid.xsize = nlon;
+  grid.ysize = nlat;
+
+  if (withBounds)
+    {
+      grid.xbounds.resize(2);
+      grid.xbounds[0] = lon1;
+      grid.xbounds[1] = lon2;
+      grid.ybounds.resize(2 * nlat);
+      grid.ybounds[0] = lat1;
+      grid.ybounds[1] = lat2;
+      if (nlat > 1) grid_gen_bounds(nlat, grid.yvals, grid.ybounds);
+    }
 }
 
 static void
 gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1, double lon2, double lat1, double lat2)
 {
   int gridtype = GRID_LONLAT;
-  bool lbounds = true;
+  bool withBounds = true;
 
   if (*pline != 0 && (*pline == '+' || *pline == '-') && (isdigit((int) *(pline + 1)) || ispunct((int) *(pline + 1))))
     {
       char *endptr = (char *) pline;
-      const auto off = strtod(pline, &endptr);
+      auto off = strtod(pline, &endptr);
       pline = endptr;
 
       lon1 -= off;
       lon2 += off;
       lat1 -= off;
       lat2 += off;
-      if (lat1 < -90) lat1 = -90;
-      if (lat2 > 90) lat2 = 90;
+      lat1 = std::max(lat1, -90.0);
+      lat2 = std::min(lat2, 90.0);
     }
 
   if (*pline != 0)
@@ -133,7 +188,7 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
               pline++;
               if (*pline == '0')
                 {
-                  lbounds = false;
+                  withBounds = false;
                   pline++;
                 }
             }
@@ -143,28 +198,29 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
               pline++;
               if (*pline == '0')
                 {
-                  lbounds = false;
+                  withBounds = false;
                   pline++;
                 }
             }
           if (*pline != 0) return;
         }
 
-      if (inc < 1e-9) inc = 1;
+      if (inc < 1.0e-9) cdo_abort("LonLat increment >%g< to small (min=1.0e-9)!", inc);
+      if (inc > 180.0) cdo_abort("LonLat increment >%g< out if range (max=180)!", inc);
     }
 
   grid.type = gridtype;
 
-  if (lon1 >= lon2 || lat1 >= lat2) cdoAbort("Invalid grid box: lon1=%g lon2=%g lat1=%g lat2=%g", lon1, lon2, lat1, lat2);
+  if (lon1 >= lon2 || lat1 >= lat2) cdo_abort("Invalid grid box: lon1=%g lon2=%g lat1=%g lat2=%g", lon1, lon2, lat1, lat2);
 
-  const auto nlon = (size_t)((lon2 - lon1) / inc + 0.5);
-  const auto nlat = (size_t)((lat2 - lat1) / inc + 0.5);
+  auto nlon = (size_t) ((lon2 - lon1) / inc + 0.5);
+  auto nlat = (size_t) ((lat2 - lat1) / inc + 0.5);
 
   grid.xvals.resize(nlon);
   grid.yvals.resize(nlat);
 
-  for (size_t i = 0; i < nlon; ++i) grid.xvals[i] = lon1 + inc / 2 + i * inc;
-  for (size_t i = 0; i < nlat; ++i) grid.yvals[i] = lat1 + inc / 2 + i * inc;
+  for (size_t i = 0; i < nlon; ++i) grid.xvals[i] = lon1 + inc * 0.5 + i * inc;
+  for (size_t i = 0; i < nlat; ++i) grid.yvals[i] = lat1 + inc * 0.5 + i * inc;
 
   if (gridtype == GRID_LONLAT)
     {
@@ -175,7 +231,7 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
     {
       std::vector<double> yvals(nlat);
       for (size_t j = 0; j < nlat; ++j) yvals[j] = grid.yvals[j];
-      const auto gridsize = nlon * nlat;
+      auto gridsize = nlon * nlat;
       grid.xvals.resize(gridsize);
       grid.yvals.resize(gridsize);
       for (size_t j = 0; j < nlat; ++j)
@@ -194,10 +250,10 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
         {
           grid.xsize = gridsize;
           grid.ysize = gridsize;
-          if (lbounds) grid.nvertex = 4;
+          if (withBounds) grid.nvertex = 4;
         }
 
-      if (lbounds && nlon > 1 && nlat > 1)
+      if (withBounds && nlon > 1 && nlat > 1)
         {
           std::vector<double> xbounds(2 * nlon), ybounds(2 * nlat);
 
@@ -213,13 +269,56 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
     }
 }
 
+static void
+gen_grid_dcw(GridDesciption &grid, const char *pline, double inc)
+{
+  auto param1 = pline;
+  auto param2 = strstr(pline, "_");
+  auto param1len = param2 ? param2 - pline : strlen(pline);
+
+  if (param2)
+    {
+      pline = param2 + 1;
+
+      if (*pline == 0) return;
+      if (!isdigit((int) *pline) && !ispunct((int) *pline)) return;
+
+      char *endptr = (char *) pline;
+      inc = strtod(pline, &endptr);
+    }
+
+  const std::string codeNames(string_to_upper({ param1, param1len }));
+
+  DCW_Lists dcw_lists;
+  if (dcw_load_lists(dcw_lists)) cdo_abort("dcw_load_lists failed!");
+
+  auto codeList = split_string(codeNames, "\\+");
+
+  dcw_sort_countries(dcw_lists);
+
+  codeList = dcw_expand_code_list(dcw_lists, codeList);
+
+  Region region;
+  if (dcw_get_region(dcw_lists, codeList, region)) cdo_abort("dcw_get_region failed!");
+
+  // printf("lon1, lon2, lat1, lat2 %g %g %g %g\n", region.west, region.east, region.south, region.north);
+  auto lon1 = std::round(region.west / inc - 0.5) * inc;
+  auto lon2 = std::round(region.east / inc + 0.5) * inc;
+  auto lat1 = std::round(region.south / inc - 0.5) * inc;
+  auto lat2 = std::round(region.north / inc + 0.5) * inc;
+  // printf("lon1, lon2, lat1, lat2 %g %g %g %g\n", lon1, lon2, lat1, lat2);
+
+  const char *param = param2 ? param2 : "";
+  gen_grid_lonlat(grid, param, inc, lon1 - inc * 0.5, lon2 + inc * 0.5, lat1 - inc * 0.5, lat2 + inc * 0.5);
+}
+
 static void
 gen_grid_gme(GridDesciption &grid, const char *pline)
 {
   if (isdigit((int) *pline))
     {
       char *endptr = (char *) pline;
-      const long ni = strtol(pline, &endptr, 10);
+      auto ni = strtol(pline, &endptr, 10);
       if (*endptr == 0)
         {
           grid.type = GRID_GME;
@@ -231,14 +330,47 @@ gen_grid_gme(GridDesciption &grid, const char *pline)
     }
 }
 
+static void
+gen_grid_healpix(GridDesciption &grid, const char *pline, bool isZoom = false)
+{
+  if (isdigit((int) *pline))
+    {
+      char *endptr = (char *) pline;
+      size_t param = strtol(pline, &endptr, 10);
+      size_t nside = isZoom ? std::lround(std::pow(2.0, param)) : param;
+      if (isZoom && *endptr != 0) return;
+      if (*endptr == 0 || endptr != pline)
+        {
+
+          HpOrder hpOrder(HpOrder::Nested);
+
+          pline = endptr;
+
+          if (*pline == '_')
+            {
+              pline++;
+              hpOrder = hp_get_order(pline);
+              if (hpOrder == HpOrder::Undef || hpOrder == HpOrder::XY) return;
+            }
+
+          size_t ncells = 12 * nside * nside;
+          grid.type = GRID_PROJECTION;
+          grid.size = ncells;
+          grid.projection = "healpix";
+          grid.healpixNside = nside;
+          grid.healpixOrder = (hpOrder == HpOrder::Ring) ? "ring" : "nested";
+        }
+    }
+}
+
 void
-gaussianLatitudesInDegrees(std::vector<double> &lats, std::vector<double> &lat_bounds, size_t nlat)
+gaussian_latitudes_in_degrees(std::vector<double> &lats, std::vector<double> &lat_bounds, size_t nlat)
 {
   // lats(nlat)
   // lat_bounds(nlat+1)
   std::vector<double> latw(nlat), latw_cumsum(nlat);
 
-  gaussianLatitudes(lats.data(), latw.data(), nlat);
+  gaussian_latitudes(nlat, lats.data(), latw.data());
 
   for (size_t j = 0; j < nlat; ++j) lats[j] = RAD2DEG * std::asin(lats[j]);
 
@@ -258,10 +390,10 @@ gen_grid_gea(GridDesciption &grid, const char *pline)
   if (isdigit((int) *pline))
     {
       auto endptr = (char *) pline;
-      const auto dx = strtod(pline, &endptr);
+      auto dx = strtod(pline, &endptr);
       if (*endptr != 0) return;
 
-      const auto dy = dx;
+      auto dy = dx;
       constexpr auto re = 6378.137;
       constexpr auto f = 1.0 / 298.257223563;
       constexpr auto rp = re * (1.0 - f);
@@ -272,7 +404,7 @@ gen_grid_gea(GridDesciption &grid, const char *pline)
       if (nlat % 2) nlat++;
 
       std::vector<double> lats(nlat), lat_bounds(nlat + 1);
-      gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+      gaussian_latitudes_in_degrees(lats, lat_bounds, nlat);
 
       std::vector<double> cell_height(nlat);
       for (size_t j = 0; j < nlat; ++j) cell_height[j] = 0.25 * polar_circumference * (lat_bounds[j] - lat_bounds[j + 1]) / 90.0;
@@ -284,9 +416,9 @@ gen_grid_gea(GridDesciption &grid, const char *pline)
       size_t ncells = 0;
       for (size_t j = 0; j < nlat; ++j)
         {
-          const auto rlat = re * std::cos(DEG2RAD * lats[j]);
-          const auto circumference = 2.0 * M_PI * rlat;
-          const auto dx_to_use = dx * dy / cell_height[j];
+          auto rlat = re * std::cos(DEG2RAD * lats[j]);
+          auto circumference = 2.0 * M_PI * rlat;
+          auto dx_to_use = dx * dy / cell_height[j];
 
           size_t nlon = std::max((int) std::lround(circumference / dx_to_use), 1);
           if (nlon % 2) nlon++;
@@ -301,7 +433,7 @@ gen_grid_gea(GridDesciption &grid, const char *pline)
       size_t ij = 0;
       for (size_t j = 0; j < nlat; ++j)
         {
-          const size_t nlon = reducedPoints[j];
+          size_t nlon = reducedPoints[j];
           for (size_t i = 0; i < nlon; ++i) lons[ij++] = i * 360. / nlon;
         }
 
@@ -322,6 +454,66 @@ gen_grid_gea(GridDesciption &grid, const char *pline)
     }
 }
 
+static void
+gen_grid_zonal(GridDesciption &grid, const char *pline)
+{
+  if (isdigit((int) *pline))
+    {
+      constexpr size_t nextra = 1;
+      grid.type = GRID_UNSTRUCTURED;
+      auto nlats = (size_t) atol(pline);
+      auto nlons = nlats * 2;
+      auto gridsize = nlats;
+      grid.size = gridsize;
+      grid.xsize = gridsize;
+      grid.ysize = gridsize;
+      grid.xvals.resize(gridsize);
+      grid.yvals.resize(gridsize);
+      for (size_t i = 0; i < nlats; ++i) grid.xvals[i] = 180.0;
+      auto dlat = 180.0 / nlats;
+      auto dlon = 360.0 / nlons;
+      // printf("dlat %g dlon %g\n", dlat, dlon);
+      for (size_t i = 0; i < nlats; ++i) grid.yvals[i] = -90.0 + i * dlat + dlat / 2.0;
+      auto nv = (nlons + 1) * 2;
+      grid.nvertex = nv;
+      grid.xbounds.resize(nv * gridsize);
+      grid.ybounds.resize(nv * gridsize);
+      std::vector<double> xbounds(nlons + 1), ybounds(nlats + 1);
+      for (size_t i = 0; i <= nlons; ++i) xbounds[i] = 0.0 + i * dlon;
+      for (size_t i = 0; i <= nlats; ++i) ybounds[i] = -90.0 + i * dlat;
+      // for (size_t i = 0; i <= nlons; ++i)  printf("lon %zu %g\n", i, xbounds[i]);
+      // for (size_t i = 0; i <= nlats; ++i)  printf("lat %zu %g\n", i, ybounds[i]);
+      size_t k = 0;
+      for (size_t j = 0; j < nlats; ++j)
+        {
+          for (size_t i = nlons; i > 0; i--)
+            {
+              grid.xbounds[k] = xbounds[i];
+              grid.ybounds[k] = ybounds[j + 1];
+              k++;
+            }
+          for (size_t i = 0; i < nextra; ++i)
+            {
+              grid.xbounds[k] = xbounds[0];
+              grid.ybounds[k] = ybounds[j + 1];
+              k++;
+            }
+          for (size_t i = 0; i < nlons; ++i)
+            {
+              grid.xbounds[k] = xbounds[i];
+              grid.ybounds[k] = ybounds[j];
+              k++;
+            }
+          for (size_t i = 0; i < nextra; ++i)
+            {
+              grid.xbounds[k] = xbounds[nlons];
+              grid.ybounds[k] = ybounds[j];
+              k++;
+            }
+        }
+    }
+}
+
 static void
 gen_grid_reg2d(GridDesciption &grid, const char *pline)
 {
@@ -372,7 +564,7 @@ gen_grid_point(GridDesciption &grid, const char *pline)
 }
 
 int
-gridFromName(const char *gridnameptr)
+grid_from_name(const char *gridnameptr)
 {
   const char *pline;
   int gridID = CDI_UNDEFID;
@@ -381,7 +573,7 @@ gridFromName(const char *gridnameptr)
   char *endptr;
 
   char *gridname = strdup(gridnameptr);
-  cstrToLowerCase(gridname);
+  cstr_to_lower(gridname);
 
   if (gridname[0] == 't')  // t<RES>grid or t<RES>spec
     {
@@ -431,9 +623,9 @@ gridFromName(const char *gridnameptr)
               grid.yvals.resize(grid.ysize);
               grid.ybounds.resize(grid.ysize * 2);
 
-              const auto nlat = grid.ysize;
+              auto nlat = grid.ysize;
               std::vector<double> lats(nlat), lat_bounds(nlat + 1);
-              gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+              gaussian_latitudes_in_degrees(lats, lat_bounds, nlat);
 
               for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
               for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2 + 1] = lat_bounds[j];
@@ -443,46 +635,51 @@ gridFromName(const char *gridnameptr)
     }
   else if (gridname[0] == 'r')  // r<LON>x<LAT>; regular 2D grid
     {
-      pline = &gridname[1];
-      gen_grid_reg2d(grid, pline);
+      gen_grid_reg2d(grid, &gridname[1]);
     }
   else if (cdo_cmpstrLenRhs(gridname, "lon=", len))  // lon=<LON>_lat=<LAT>; one gridpoint
     {
-      pline = &gridname[len];
-      gen_grid_point(grid, pline);
+      gen_grid_point(grid, &gridname[len]);
     }
   else if (gridname[0] == 'g' && gridname[1] == 'm' && gridname[2] == 'e')  // gme<NI>
     {
-      pline = &gridname[3];
-      gen_grid_gme(grid, pline);
+      gen_grid_gme(grid, &gridname[3]);
     }
   else if (gridname[0] == 'n' && gridname[1] == 'i')  // ni<NI>
     {
-      pline = &gridname[2];
-      gen_grid_gme(grid, pline);
+      gen_grid_gme(grid, &gridname[2]);
+    }
+  else if (gridname[0] == 'h' && gridname[1] == 'p' && gridname[2] == 'z')  // healpix  hpz<zoom>[_order] (order=[nested|ring|xy])
+    {
+      gen_grid_healpix(grid, &gridname[3], true);
+    }
+  else if (gridname[0] == 'h' && gridname[1] == 'p')  // healpix  hp<nside>[_order] (order=[nested|ring|xy])
+    {
+      gen_grid_healpix(grid, &gridname[2]);
     }
   else if (gridname[0] == 'g' && gridname[1] == 'e' && gridname[2] == 'a')
     {
-      // gea<DX>: gaussian reduced equal area; DX in km
-      pline = &gridname[3];
-      gen_grid_gea(grid, pline);
+      gen_grid_gea(grid, &gridname[3]);  // gea<DX>: gaussian reduced equal area; DX in km
     }
   else if ((gridname[0] == 'f' || gridname[0] == 'n') && isdigit((int) gridname[1]))
     {
       // FXXX - full (regular) Gaussian grid with XXX latitude lines between the pole and equator
       pline = &gridname[1];
-      const auto numLPE = strtol(pline, &endptr, 10);
+      auto numLPE = strtol(pline, &endptr, 10);
       pline = endptr;
 
+      if (*pline == 'b')
+        {
+          grid.genBounds = true;
+          pline++;
+        }
+
+      if (*pline == '_') pline++;
+
       if (cdo_cmpstrLenRhs(pline, "zon", len))
         {
           grid.xsize = 1;
-          pline += 3;
-        }
-      else if (*pline == 'b')
-        {
-          grid.genBounds = true;
-          pline++;
+          pline += len;
         }
 
       if (*pline == 0)
@@ -500,7 +697,7 @@ gridFromName(const char *gridnameptr)
 
           size_t nlat = grid.ysize;
           std::vector<double> lats(nlat), lat_bounds(nlat + 1);
-          gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+          gaussian_latitudes_in_degrees(lats, lat_bounds, nlat);
 
           for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
           for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2 + 1] = lat_bounds[j];
@@ -511,18 +708,21 @@ gridFromName(const char *gridnameptr)
   else if (gridname[0] == 'o' && isdigit((int) gridname[1]))  // O<xxx>
     {
       pline = &gridname[1];
-      const auto numLPE = strtol(pline, &endptr, 10);
+      auto numLPE = strtol(pline, &endptr, 10);
       pline = endptr;
 
+      if (*pline == 'b')
+        {
+          grid.genBounds = true;
+          pline++;
+        }
+
+      if (*pline == '_') pline++;
+
       if (cdo_cmpstrLenRhs(pline, "zon", len))
         {
           grid.xsize = 1;
-          pline += 3;
-        }
-      else if (*pline == 'b')
-        {
-          grid.genBounds = true;
-          pline++;
+          pline += len;
         }
 
       if (*pline == 0)
@@ -557,33 +757,40 @@ gridFromName(const char *gridnameptr)
             }
         }
     }
+  else if (cdo_cmpstrLenRhs(gridname, "dcw:", len))  // dcw:code_Xdeg
+    {
+      gen_grid_dcw(grid, &gridname[len], 0.1);
+    }
   else if (cdo_cmpstrLenRhs(gridname, "germany", len))  // germany_Xdeg
     {
-      pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, 0.1, 5.6, 15.2, 47.1, 55.1);
+      gen_grid_lonlat(grid, &gridname[len], 0.1, 5.6, 15.2, 47.1, 55.1);
     }
   else if (cdo_cmpstrLenRhs(gridname, "europe", len))  // europe_Xdeg
     {
-      pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, 1, -30, 60, 30, 80);
+      gen_grid_lonlat(grid, &gridname[len], 1, -30, 60, 30, 80);
     }
   else if (cdo_cmpstrLenRhs(gridname, "africa", len))  // africa_Xdeg
     {
-      pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, 1, -20, 60, -40, 40);
+      gen_grid_lonlat(grid, &gridname[len], 1, -20, 60, -40, 40);
     }
   else if (cdo_cmpstrLenRhs(gridname, "global", len))  // global_Xdeg
     {
-      pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, 1, -180, 180, -90, 90);
+      gen_grid_lonlat(grid, &gridname[len], 1, -180, 180, -90, 90);
+    }
+  else if (cdo_cmpstrLenRhs(gridname, "zonal", len))  // zonal_Xdeg
+    {
+      gen_grid_zonal(grid, &gridname[len], 1, -180, 180, -90, 90);
+    }
+  else if (gridname[0] == 'z')  // z<LAT>; zonal unstructured grid with <LAT> latitudes
+    {
+      gen_grid_zonal(grid, &gridname[1]);
     }
   else if (cdo_cmpstrLenRhs(gridname, "ico", len))  // icoR02BXX
     {
-      pline = &gridname[len];
-      gen_grid_icosphere(grid, pline);
+      gen_grid_icosphere(grid, &gridname[len]);
     }
 
-  if (grid.type != -1) gridID = gridDefine(grid);
+  if (grid.type != -1) gridID = grid_define(grid);
 
   free(gridname);
 
