@@ -14,8 +14,9 @@
 #include "util_string.h"
 
 static void
-printDblsPrefixAutoBrk(FILE *fp, int dig, const char prefix[], int nbyte0, size_t n, const double vals[])
+printDblsPrefixAutoBrk(FILE *fp, int dig, const char *prefix, size_t n, const double vals[])
 {
+  int nbyte0 = strlen(prefix);
   fputs(prefix, fp);
   auto nbyte = nbyte0;
   for (size_t i = 0; i < n; ++i)
@@ -31,8 +32,9 @@ printDblsPrefixAutoBrk(FILE *fp, int dig, const char prefix[], int nbyte0, size_
 }
 
 static void
-printIntsPrefixAutoBrk(FILE *fp, const char prefix[], int nbyte0, size_t n, const int vals[])
+printIntsPrefixAutoBrk(FILE *fp, const char *prefix, size_t n, const int vals[])
 {
+  int nbyte0 = strlen(prefix);
   fputs(prefix, fp);
   auto nbyte = nbyte0;
   for (size_t i = 0; i < n; ++i)
@@ -48,20 +50,22 @@ printIntsPrefixAutoBrk(FILE *fp, const char prefix[], int nbyte0, size_t n, cons
 }
 
 static void
-printBounds(FILE *fp, int dig, const char prefix[], size_t nbyte0, size_t n, size_t nvertex, const double bounds[])
+print_bounds(FILE *fp, int dig, const char *prefix, size_t n, size_t nvertex, const double bounds[])
 {
+  int nbyte0 = strlen(prefix);
   fputs(prefix, fp);
   for (size_t i = 0; i < n; ++i)
     {
-      if (i > 0) fprintf(fp, "\n%*s", (int) nbyte0, "");
+      if (i > 0) fprintf(fp, "\n%*s", nbyte0, "");
       for (size_t iv = 0; iv < nvertex; iv++) fprintf(fp, "%.*g ", dig, bounds[i * nvertex + iv]);
     }
   fputs("\n", fp);
 }
 
 static void
-printMask(FILE *fp, const char prefix[], size_t nbyte0, size_t n, const int mask[])
+printMask(FILE *fp, const char *prefix, size_t n, const int mask[])
 {
+  int nbyte0 = strlen(prefix);
   fputs(prefix, fp);
   auto nbyte = nbyte0;
   for (size_t i = 0; i < n; ++i)
@@ -76,17 +80,6 @@ printMask(FILE *fp, const char prefix[], size_t nbyte0, size_t n, const int mask
   fputs("\n", fp);
 }
 
-static inline void *
-resizeBuffer(void **buf, size_t *bufSize, size_t reqSize)
-{
-  if (reqSize > *bufSize)
-    {
-      *buf = Realloc(*buf, reqSize);
-      *bufSize = reqSize;
-    }
-  return *buf;
-}
-
 static void
 grid_print_attributes(FILE *fp, int gridID)
 {
@@ -94,8 +87,6 @@ grid_print_attributes(FILE *fp, int gridID)
   int varID = CDI_GLOBAL;
   int atttype, attlen;
   char attname[CDI_MAX_NAME + 1];
-  void *attBuf = nullptr;
-  size_t attBufSize = 0;
   char fltstr[128];
 
   int natts;
@@ -110,30 +101,27 @@ grid_print_attributes(FILE *fp, int gridID)
 
       if (atttype == CDI_DATATYPE_TXT)
         {
-          const auto attSize = (size_t)(attlen + 1) * sizeof(char);
-          auto atttxt = (char *) resizeBuffer(&attBuf, &attBufSize, attSize);
-          cdiInqAttTxt(cdiID, varID, attname, attlen, atttxt);
+          std::vector<char> atttxt(attlen + 1);
+          cdiInqAttTxt(cdiID, varID, attname, attlen, atttxt.data());
           atttxt[attlen] = 0;
-          if (strchr(atttxt, '"'))
-            fprintf(fp, "%s = '%s'\n", attname, atttxt);
+          if (strchr(atttxt.data(), '"'))
+            fprintf(fp, "%s = '%s'\n", attname, atttxt.data());
           else
-            fprintf(fp, "%s = \"%s\"\n", attname, atttxt);
+            fprintf(fp, "%s = \"%s\"\n", attname, atttxt.data());
         }
       else if (atttype == CDI_DATATYPE_INT8 || atttype == CDI_DATATYPE_UINT8 || atttype == CDI_DATATYPE_INT16
                || atttype == CDI_DATATYPE_UINT16 || atttype == CDI_DATATYPE_INT32 || atttype == CDI_DATATYPE_UINT32)
         {
-          const auto attSize = (size_t) attlen * sizeof(int);
-          auto attint = (int *) resizeBuffer(&attBuf, &attBufSize, attSize);
-          cdiInqAttInt(cdiID, varID, attname, attlen, &attint[0]);
+          std::vector<int> attint(attlen);
+          cdiInqAttInt(cdiID, varID, attname, attlen, attint.data());
           fprintf(fp, "%s =", attname);
           for (int i = 0; i < attlen; ++i) fprintf(fp, " %d", attint[i]);
           fprintf(fp, "\n");
         }
       else if (atttype == CDI_DATATYPE_FLT32 || atttype == CDI_DATATYPE_FLT64)
         {
-          const auto attSize = (size_t) attlen * sizeof(double);
-          auto attflt = (double *) resizeBuffer(&attBuf, &attBufSize, attSize);
-          cdiInqAttFlt(cdiID, varID, attname, attlen, attflt);
+          std::vector<double> attflt(attlen);
+          cdiInqAttFlt(cdiID, varID, attname, attlen, attflt.data());
           fprintf(fp, "%s =", attname);
           if (atttype == CDI_DATATYPE_FLT32)
             for (int i = 0; i < attlen; ++i)
@@ -144,27 +132,24 @@ grid_print_attributes(FILE *fp, int gridID)
           fprintf(fp, "\n");
         }
     }
-
-  Free(attBuf);
 }
 
 static void
 grid_print_kernel(int gridID, int opt, FILE *fp)
 {
   size_t xdim, ydim;
-  char attstr[CDI_MAX_NAME], attstr2[CDI_MAX_NAME];
-  const auto nxvals = gridInqXvals(gridID, nullptr);
-  const auto nyvals = gridInqYvals(gridID, nullptr);
-  const auto nxbounds = gridInqXbounds(gridID, nullptr);
-  const auto nybounds = gridInqYbounds(gridID, nullptr);
+  auto nxvals = gridInqXvals(gridID, nullptr);
+  auto nyvals = gridInqYvals(gridID, nullptr);
+  auto nxbounds = gridInqXbounds(gridID, nullptr);
+  auto nybounds = gridInqYbounds(gridID, nullptr);
 
-  const auto gridsize = gridInqSize(gridID);
-  const auto xsize = gridInqXsize(gridID);
-  const auto ysize = gridInqYsize(gridID);
-  const auto type = gridInqType(gridID);
-  const auto nvertex = gridInqNvertex(gridID);
-  const auto xstrlen = gridInqXIsc(gridID);
-  const auto ystrlen = gridInqYIsc(gridID);
+  auto gridsize = gridInqSize(gridID);
+  auto xsize = gridInqXsize(gridID);
+  auto ysize = gridInqYsize(gridID);
+  auto type = gridInqType(gridID);
+  auto nvertex = gridInqNvertex(gridID);
+  auto xstrlen = gridInqXIsc(gridID);
+  auto ystrlen = gridInqYIsc(gridID);
   int datatype = CDI_UNDEFID;
   cdiInqKeyInt(gridID, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
 
@@ -202,7 +187,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
           }
     }
 
-  const int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
+  int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
 
   fprintf(fp, "gridtype  = %s\n", gridNamePtr(type));
   fprintf(fp, "gridsize  = %zu\n", gridsize);
@@ -218,41 +203,32 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
       if (nxvals > 0 || xcvals)
         {
-          int length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, attstr, &length);
-          if (attstr[0]) fprintf(fp, "xname     = %s\n", attstr);
-          length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_DIMNAME, attstr2, &length);
-          if (attstr2[0] && !cdo_cmpstr(attstr, attstr2)) fprintf(fp, "xdimname  = %s\n", attstr2);
-          length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_LONGNAME, attstr, &length);
-          if (attstr[0]) fprintf(fp, "xlongname = \"%s\"\n", attstr);
-          length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, attstr, &length);
-          if (attstr[0]) fprintf(fp, "xunits    = \"%s\"\n", attstr);
+          auto name = cdo::inq_key_string(gridID, CDI_XAXIS, CDI_KEY_NAME);
+          auto dimname = cdo::inq_key_string(gridID, CDI_XAXIS, CDI_KEY_DIMNAME);
+          auto longname = cdo::inq_key_string(gridID, CDI_XAXIS, CDI_KEY_LONGNAME);
+          auto units = cdo::inq_key_string(gridID, CDI_XAXIS, CDI_KEY_UNITS);
+          if (name.size()) fprintf(fp, "xname     = %s\n", name.c_str());
+          if (dimname.size() && dimname != name) fprintf(fp, "xdimname  = %s\n", dimname.c_str());
+          if (longname.size()) fprintf(fp, "xlongname = \"%s\"\n", longname.c_str());
+          if (units.size()) fprintf(fp, "xunits    = \"%s\"\n", units.c_str());
         }
 
       if (nyvals > 0 || ycvals)
         {
-          int length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, attstr, &length);
-          if (attstr[0]) fprintf(fp, "yname     = %s\n", attstr);
-          length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_DIMNAME, attstr2, &length);
-          if (attstr2[0] && !cdo_cmpstr(attstr, attstr2)) fprintf(fp, "ydimname  = %s\n", attstr2);
-          length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_LONGNAME, attstr, &length);
-          if (attstr[0]) fprintf(fp, "ylongname = \"%s\"\n", attstr);
-          length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, attstr, &length);
-          if (attstr[0]) fprintf(fp, "yunits    = \"%s\"\n", attstr);
+          auto name = cdo::inq_key_string(gridID, CDI_YAXIS, CDI_KEY_NAME);
+          auto dimname = cdo::inq_key_string(gridID, CDI_YAXIS, CDI_KEY_DIMNAME);
+          auto longname = cdo::inq_key_string(gridID, CDI_YAXIS, CDI_KEY_LONGNAME);
+          auto units = cdo::inq_key_string(gridID, CDI_YAXIS, CDI_KEY_UNITS);
+          if (name.size()) fprintf(fp, "yname     = %s\n", name.c_str());
+          if (dimname.size() && dimname != name) fprintf(fp, "ydimname  = %s\n", dimname.c_str());
+          if (longname.size()) fprintf(fp, "ylongname = \"%s\"\n", longname.c_str());
+          if (units.size()) fprintf(fp, "yunits    = \"%s\"\n", units.c_str());
         }
 
       if (type == GRID_UNSTRUCTURED || type == GRID_CURVILINEAR)
         {
-          int length = CDI_MAX_NAME;
-          cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_VDIMNAME, attstr, &length);
-          if (attstr[0]) fprintf(fp, "vdimname  = %s\n", attstr);
+          auto vdimName = cdo::inq_key_string(gridID, CDI_GLOBAL, CDI_KEY_VDIMNAME);
+          if (vdimName.size()) fprintf(fp, "vdimname  = %s\n", vdimName.c_str());
         }
       if (type == GRID_UNSTRUCTURED && nvertex > 0) fprintf(fp, "nvertex   = %d\n", nvertex);
     }
@@ -302,9 +278,9 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
             int length = 0;
             if (CDI_NOERR == cdiInqKeyLen(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, &length))
               {
-                char reference_link[8192];
-                cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, reference_link, &length);
-                fprintf(fp, "uri       = %s\n", reference_link);
+                char referenceLink[8192];
+                cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, referenceLink, &length);
+                fprintf(fp, "uri       = %s\n", referenceLink);
               }
           }
 
@@ -318,7 +294,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
                 xinc = gridInqXinc(gridID);
               }
 
-            if (IS_NOT_EQUAL(xinc, 0) && opt)
+            if (is_not_equal(xinc, 0.0) && opt)
               {
                 fprintf(fp, "xfirst    = %.*g\n", dig, xfirst);
                 fprintf(fp, "xinc      = %.*g\n", dig, xinc);
@@ -327,8 +303,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
               {
                 std::vector<double> xvals(nxvals);
                 gridInqXvals(gridID, xvals.data());
-                static const char prefix[] = "xvals     = ";
-                printDblsPrefixAutoBrk(fp, dig, prefix, (int) sizeof(prefix) - 1, nxvals, xvals.data());
+                printDblsPrefixAutoBrk(fp, dig, "xvals     = ", nxvals, xvals.data());
               }
           }
 
@@ -336,18 +311,12 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
           {
             std::vector<double> xbounds(nxbounds);
             gridInqXbounds(gridID, xbounds.data());
-            static const char prefix[] = "xbounds   = ";
-            printBounds(fp, dig, prefix, sizeof(prefix) - 1, xdim, nvertex, xbounds.data());
+            print_bounds(fp, dig, "xbounds   = ", xdim, nvertex, xbounds.data());
           }
 
         if (xcvals)
           {
-            int length = CDI_MAX_NAME;
-            cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, attstr, &length);
-            if (attstr[0])
-              fprintf(fp, "x%ss  = \"%.*s\"", attstr, xstrlen, xcvals[0]);
-            else
-              fprintf(fp, "xstrings  = \"%.*s\"", xstrlen, xcvals[0]);
+            fprintf(fp, "xcvals    = \"%.*s\"", xstrlen, xcvals[0]);
             for (size_t i = 1; i < xsize; ++i) fprintf(fp, ", \"%.*s\"", xstrlen, xcvals[i]);
             fprintf(fp, "\n");
 
@@ -365,7 +334,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
                 yinc = gridInqYinc(gridID);
               }
 
-            if (IS_NOT_EQUAL(yinc, 0) && opt)
+            if (is_not_equal(yinc, 0.0) && opt)
               {
                 fprintf(fp, "yfirst    = %.*g\n", dig, yfirst);
                 fprintf(fp, "yinc      = %.*g\n", dig, yinc);
@@ -374,8 +343,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
               {
                 std::vector<double> yvals(nyvals);
                 gridInqYvals(gridID, yvals.data());
-                static const char prefix[] = "yvals     = ";
-                printDblsPrefixAutoBrk(fp, dig, prefix, (int) sizeof(prefix) - 1, nyvals, yvals.data());
+                printDblsPrefixAutoBrk(fp, dig, "yvals     = ", nyvals, yvals.data());
               }
           }
 
@@ -383,18 +351,12 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
           {
             std::vector<double> ybounds(nybounds);
             gridInqYbounds(gridID, ybounds.data());
-            static const char prefix[] = "ybounds   = ";
-            printBounds(fp, dig, prefix, sizeof(prefix) - 1, ydim, nvertex, ybounds.data());
+            print_bounds(fp, dig, "ybounds   = ", ydim, nvertex, ybounds.data());
           }
 
         if (ycvals)
           {
-            int length = CDI_MAX_NAME;
-            cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, attstr, &length);
-            if (attstr[0])
-              fprintf(fp, "x%ss  = \"%.*s\"", attstr, ystrlen, ycvals[0]);
-            else
-              fprintf(fp, "ystrings  = \"%.*s\"", ystrlen, ycvals[0]);
+            fprintf(fp, "ycvals    = \"%.*s\"", ystrlen, ycvals[0]);
             for (size_t i = 1; i < ysize; ++i) fprintf(fp, ", \"%.*s\"", ystrlen, ycvals[i]);
             fprintf(fp, "\n");
 
@@ -406,16 +368,14 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
           {
             std::vector<double> area(gridsize);
             gridInqArea(gridID, area.data());
-            static const char prefix[] = "area      = ";
-            printDblsPrefixAutoBrk(fp, dig, prefix, (int) sizeof(prefix) - 1, gridsize, area.data());
+            printDblsPrefixAutoBrk(fp, dig, "area      = ", gridsize, area.data());
           }
 
         if (type == GRID_GAUSSIAN_REDUCED)
           {
             std::vector<int> reducedPoints(ysize);
             gridInqReducedPoints(gridID, reducedPoints.data());
-            static const char prefix[] = "reducedPoints = ";
-            printIntsPrefixAutoBrk(fp, prefix, (int) sizeof(prefix) - 1, (size_t)(ysize > 0 ? ysize : 0), reducedPoints.data());
+            printIntsPrefixAutoBrk(fp, "reducedPoints = ", (size_t) (ysize > 0 ? ysize : 0), reducedPoints.data());
           }
 
 #ifdef HIRLAM_EXTENSIONS
@@ -424,16 +384,14 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
           cdiInqKeyInt(gridID, CDI_GLOBAL, CDI_KEY_SCANNINGMODE, &scanningMode);
           fprintf(fp, "scanningMode = %d\n", scanningMode);
         }
-#endif  // HIRLAM_EXTENSIONS
+#endif
 
         if (type == GRID_PROJECTION)
           {
-            int length = CDI_MAX_NAME;
-            cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, attstr, &length);
-            if (attstr[0]) fprintf(fp, "grid_mapping = %s\n", attstr);
-            length = CDI_MAX_NAME;
-            cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, attstr, &length);
-            if (attstr[0]) fprintf(fp, "grid_mapping_name = %s\n", attstr);
+            auto gridMapping = cdo::inq_key_string(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME);
+            auto gridMappingName = cdo::inq_key_string(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME);
+            if (gridMapping.size()) fprintf(fp, "grid_mapping = %s\n", gridMapping.c_str());
+            if (gridMappingName.size()) fprintf(fp, "grid_mapping_name = %s\n", gridMappingName.c_str());
             grid_print_attributes(fp, gridID);
           }
 
@@ -477,11 +435,10 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
     {
       std::vector<int> mask(gridsize);
       gridInqMask(gridID, mask.data());
-      static const char prefix[] = "mask      = ";
-      printMask(fp, prefix, sizeof(prefix) - 1, (size_t)(gridsize > 0 ? gridsize : 0), mask.data());
+      printMask(fp, "mask      = ", (size_t) (gridsize > 0 ? gridsize : 0), mask.data());
     }
 
-  const auto projID = gridInqProj(gridID);
+  auto projID = gridInqProj(gridID);
   if (projID != CDI_UNDEFID && gridInqType(projID) == GRID_PROJECTION) grid_print_kernel(projID, opt, fp);
 }
 
