@@ -47,7 +47,7 @@ cdo_omp_get_thread_num()
 #endif
 
 static long
-int_index(const long n, const Varray<double> &x1, const double x2)
+int_index(long n, const Varray<double> &x1, const double x2)
 {
   long k;
   long klo = 0;
