@@ -184,10 +184,10 @@ intlinarr2p(long nxm, long nym, double **fieldm, const Varray<double> &xm, const
   long ii, jj;
   double value = 0;
 
-  for (jj = 1; jj < nym; jj++)
+  for (jj = 1; jj < nym; ++jj)
     if (y >= std::min(ym[jj - 1], ym[jj]) && y <= std::max(ym[jj - 1], ym[jj])) break;
 
-  for (ii = 1; ii < nxm; ii++)
+  for (ii = 1; ii < nxm; ++ii)
     if (x >= xm[ii - 1] && x <= xm[ii]) break;
 
   if (jj < nym && ii < nxm)
@@ -207,7 +207,7 @@ intlinarr2(T missval, int lon_is_circular, size_t nxm, size_t nym, const Varray<
            const Varray<double> &ym, size_t gridsize2, Varray<T> &varray2, const Varray<double> &x, const Varray<double> &y)
 {
   auto nlon1 = nxm;
-  std::atomic<size_t> atomicCount{0};
+  std::atomic<size_t> atomicCount{ 0 };
 
   if (lon_is_circular) nlon1--;
   const size_t gridsize1 = nlon1 * nym;
@@ -219,8 +219,7 @@ intlinarr2(T missval, int lon_is_circular, size_t nxm, size_t nym, const Varray<
   progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(atomicCount, varray1, varray2, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, lon_is_circular, grid1_mask)
+#pragma omp parallel for default(shared)
 #endif
   for (size_t i = 0; i < gridsize2; ++i)
     {
@@ -229,7 +228,7 @@ intlinarr2(T missval, int lon_is_circular, size_t nxm, size_t nym, const Varray<
       varray2[i] = missval;
 
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double)atomicCount / gridsize2);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double) atomicCount / gridsize2);
 
       size_t ii, jj;
       auto lfound = rect_grid_search(ii, jj, x[i], y[i], nxm, nym, xm, ym);
@@ -257,7 +256,7 @@ intlinarr2(T missval, int lon_is_circular, size_t nxm, size_t nym, const Varray<
           wgts[2] = (x[i] - xm[ii]) * (y[i] - ym[jj - 1]) / ((xm[ii - 1] - xm[ii]) * (ym[jj] - ym[jj - 1]));
 
           // printf("%2ld %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f\n",
-          // dst_add, plon, plat, wgts[0], wgts[1], wgts[2], wgts[3], iw, jw);
+          // tgt_add, plon, plat, wgts[0], wgts[1], wgts[2], wgts[3], iw, jw);
 
           double fwsum = 0.0;
           for (int n = 0; n < 4; ++n) fwsum += varray1[src_add[n]] * wgts[n];
@@ -276,8 +275,8 @@ intlinarr(long nxm, double *ym, double *xm, int nx, double *y, double *x)
 
     Uwe Schulzweida  04/05/1995
   */
-  for (long jj = 1; jj < nxm; jj++)
-    for (long j = 0; j < nx; j++)
+  for (long jj = 1; jj < nxm; ++jj)
+    for (long j = 0; j < nx; ++j)
       if (x[j] >= xm[jj - 1] && x[j] <= xm[jj]) y[j] = intlin(x[j], ym[jj - 1], xm[jj - 1], ym[jj], xm[jj]);
 }
 
@@ -286,6 +285,9 @@ intgridbil(Field &field1, Field &field2)
 {
   auto gridID1 = field1.grid;
   auto gridID2 = field2.grid;
+  if (gridID1 == -1) cdo_abort("Source grid undefined!");
+  if (gridID2 == -1) cdo_abort("Target grid undefined!");
+
   const auto missval = field1.missval;
 
   auto nlon1 = gridInqXsize(gridID1);
@@ -323,12 +325,8 @@ intgridbil(Field &field1, Field &field2)
 
   if (lgeorefgrid)
     {
-      if (gridInqType(gridID2) == GRID_GME) gridID2 = gridToUnstructured(gridID2, 0);
-
-      if (gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR)
-        gridID2 = gridToCurvilinear(gridID2, 0);
-
-      if (!gridHasCoordinates(gridID2)) cdo_abort("Target grid has no coordinate values!");
+      gridID2 = generate_full_point_grid(gridID2);
+      if (!gridHasCoordinates(gridID2)) cdo_abort("Target cell center coordinates missing!");
     }
 
   const auto gridsize2 = gridInqSize(gridID2);
@@ -365,6 +363,8 @@ intgridbil(Field &field1, Field &field2)
 
   if (field2.grid != gridID2) gridDestroy(gridID2);
 
+  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
+
   if (field1.memType == MemType::Float)
     intlinarr2((float) missval, lon_is_circular, nlon1, nlat1, field1.vec_f, lons1, lats1, gridsize2, field2.vec_f, xvals2, yvals2);
   else
@@ -420,8 +420,8 @@ interpolate(Field &field1, Field &field2)
 
   if (nlon > 1)
     {
-      lon[-1] = lon[nlon - 1] - 360 > 2 * lon[0] - lon[1] ? lon[nlon - 1] - 360 : 2 * lon[0] - lon[1];
-      lon[nlon] = lon[0] + 360 < 2 * lon[nlon - 1] - lon[nlon - 2] ? lon[0] + 360 : 2 * lon[nlon - 1] - lon[nlon - 2];
+      lon[-1] = (lon[nlon - 1] - 360 > 2 * lon[0] - lon[1]) ? lon[nlon - 1] - 360 : 2 * lon[0] - lon[1];
+      lon[nlon] = (lon[0] + 360 < 2 * lon[nlon - 1] - lon[nlon - 2]) ? lon[0] + 360 : 2 * lon[nlon - 1] - lon[nlon - 2];
     }
   else
     {
@@ -445,8 +445,8 @@ interpolate(Field &field1, Field &field2)
   if (lat[nlat] < -90) lat[nlat] = -99;
   if (lat[nlat] > 90) lat[nlat] = 99;
 
-  Varray<double> lono_array(out_nlon < 2 ? 4 : out_nlon + 2);
-  Varray<double> lato_array(out_nlat < 2 ? 4 : out_nlat + 2);
+  Varray<double> lono_array((out_nlon < 2) ? 4 : out_nlon + 2);
+  Varray<double> lato_array((out_nlat < 2) ? 4 : out_nlat + 2);
   double *lono = lono_array.data() + 1;
   double *lato = lato_array.data() + 1;
 
@@ -457,10 +457,10 @@ interpolate(Field &field1, Field &field2)
   cdo_grid_to_degree(gridIDo, CDI_XAXIS, out_nlon, lono, "grid2 center lon");
   cdo_grid_to_degree(gridIDo, CDI_YAXIS, out_nlat, lato, "grid2 center lat");
 
-  for (i = 0; i < out_nlon - 1; i++)
+  for (i = 0; i < out_nlon - 1; ++i)
     if (lono[i + 1] <= lono[i]) break;
 
-  for (i++; i < out_nlon; i++)
+  for (i++; i < out_nlon; ++i)
     {
       lono[i] += 360;
       if (i < out_nlon - 1 && lono[i + 1] + 360 <= lono[i]) cdo_abort("Longitudes of output grid are not in ascending order!");
@@ -473,7 +473,7 @@ interpolate(Field &field1, Field &field2)
       cdo_abort("Latitudes of output grid must be between 90 and -90!");
     }
 
-  for (i = 0; i < out_nlat - 1; i++)
+  for (i = 0; i < out_nlat - 1; ++i)
     if (IS_EQUAL(lato[i + 1], lato[i]) || (i < out_nlat - 2 && ((lato[i + 1] > lato[i]) != (lato[i + 2] > lato[i + 1]))))
       {
         cdo_abort("Latitudes of output grid must be in descending or ascending order!");
@@ -481,9 +481,9 @@ interpolate(Field &field1, Field &field2)
 
   if (out_nlon > 1)
     {
-      lono[-1] = lono[out_nlon - 1] - 360 > 2 * lono[0] - lono[1] ? lono[out_nlon - 1] - 360 : 2 * lono[0] - lono[1];
-      lono[out_nlon] = lono[0] + 360 < 2 * lono[out_nlon - 1] - lono[out_nlon - 2] ? lono[0] + 360
-                                                                                   : 2 * lono[out_nlon - 1] - lono[out_nlon - 2];
+      lono[-1] = (lono[out_nlon - 1] - 360 > 2 * lono[0] - lono[1]) ? lono[out_nlon - 1] - 360 : 2 * lono[0] - lono[1];
+      lono[out_nlon] = (lono[0] + 360 < 2 * lono[out_nlon - 1] - lono[out_nlon - 2]) ? lono[0] + 360
+                                                                                     : 2 * lono[out_nlon - 1] - lono[out_nlon - 2];
     }
   else
     {
@@ -571,16 +571,16 @@ interpolate(Field &field1, Field &field2)
         {
           if (volat1 > volat2)
             {
-              for (l1 = 0; l1 < nxlat && xlat[l1] > volat1; l1++)
+              for (l1 = 0; l1 < nxlat && xlat[l1] > volat1; ++l1)
                 ;
-              for (l2 = l1; l2 < nxlat && xlat[l2] > volat2; l2++)
+              for (l2 = l1; l2 < nxlat && xlat[l2] > volat2; ++l2)
                 ;
             }
           else
             {
-              for (l1 = 0; l1 < nxlat && xlat[l1] > volat2; l1++)
+              for (l1 = 0; l1 < nxlat && xlat[l1] > volat2; ++l1)
                 ;
-              for (l2 = l1; l2 < nxlat && xlat[l2] > volat1; l2++)
+              for (l2 = l1; l2 < nxlat && xlat[l2] > volat1; ++l2)
                 ;
             }
         }
@@ -588,16 +588,16 @@ interpolate(Field &field1, Field &field2)
         {
           if (volat1 < volat2)
             {
-              for (l1 = 0; l1 < nxlat && xlat[l1] < volat1; l1++)
+              for (l1 = 0; l1 < nxlat && xlat[l1] < volat1; ++l1)
                 ;
-              for (l2 = l1; l2 < nxlat && xlat[l2] < volat2; l2++)
+              for (l2 = l1; l2 < nxlat && xlat[l2] < volat2; ++l2)
                 ;
             }
           else
             {
-              for (l1 = 0; l1 < nxlat && xlat[l1] < volat2; l1++)
+              for (l1 = 0; l1 < nxlat && xlat[l1] < volat2; ++l1)
                 ;
-              for (l2 = l1; l2 < nxlat && xlat[l2] < volat1; l2++)
+              for (l2 = l1; l2 < nxlat && xlat[l2] < volat1; ++l2)
                 ;
             }
         }
@@ -714,7 +714,7 @@ interpolate(Field &field1, Field &field2)
         {
           double sum = 0.0;
           double wsum = 0.0;
-          for (k = 0; k < 2; k++)
+          for (k = 0; k < 2; ++k)
             {
               if (k == 0)
                 {
@@ -775,14 +775,11 @@ interpolate(Field &field1, Field &field2)
                         }
                       else
                         {
-                          vlon1 = (volon1 <= vilon1 ? vilon1 : volon1);
-                          vlon2 = (vilon2 <= volon2 ? vilon2 : volon2);
-                          vlat1 = (volat1 <= vilat1 ? vilat1 : volat1);
-                          vlat2 = (vilat2 <= volat2 ? vilat2 : volat2);
-                          if (vlon1 >= vlon2 - (volon2 - volon1) * 1e-5 || vlat1 >= vlat2 - (volat2 - volat1) * 1e-5)
-                            {
-                              continue;
-                            }
+                          vlon1 = ((volon1 <= vilon1) ? vilon1 : volon1);
+                          vlon2 = ((vilon2 <= volon2) ? vilon2 : volon2);
+                          vlat1 = ((volat1 <= vilat1) ? vilat1 : volat1);
+                          vlat2 = ((vilat2 <= volat2) ? vilat2 : volat2);
+                          if (vlon1 >= vlon2 - (volon2 - volon1) * 1e-5 || vlat1 >= vlat2 - (volat2 - volat1) * 1e-5) { continue; }
                           faclon1 = (vlon1 - vilon1) / (vilon2 - vilon1);
                           faclon2 = (vlon2 - vilon1) / (vilon2 - vilon1);
                           faclat1 = (vlat1 - vilat1) / (vilat2 - vilat1);
