@@ -1,18 +1,9 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
+          Oliver Heidmann
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 #include "modules.h"
@@ -22,55 +13,110 @@
 #include <cstring>
 #include <algorithm>  // for std::sort()
 
+int add_alias(const std::string &alias, const std::string &original);
+void add_module(const std::string &module_name, const module_t &new_module);
+
 /* removes '-' from operator string and returns copy of parameter operatorCommand */
 /***
   vector for library handles for loaded custom modules
   */
 std::vector<void *> custom_modules_lib_handles;
 
-/***
-  Maps operator names to their module names
- */
-std::map<std::string, std::string> modules_map;
-
 /***
   Contains added modules as values and their names as key
   */
-std::map<std::string, module_t> modules;
 
-/***
-  Key: operator alias / Value: operator original name
- */
-std::map<std::string, std::string> aliases;
-static const char *
-getOperatorName(const char *operatorCommand)
+Alias::Alias(const std::string &_alias, const std::string &_original) : alias(_alias), original(_original) {}
+
+// clang-format off
+module_t::module_t(const std::string &p_mod_name,
+                   void *(*p_func)(void *),
+                   const char **p_help,
+                   const std::vector<std::string> &p_opers,
+                   short p_m, short p_n, short p_siC, short p_soC,
+                   PositionRestrictions p_pos_restriction,
+                   const std::vector<Alias> &p_aliases)
+
+                 : mod_name(p_mod_name),
+                   func(p_func),
+                   help(p_help),
+                   mode(p_m),
+                   number(p_n),
+                   operators(p_opers),
+                   constraints({p_siC, p_soC, p_pos_restriction}),
+                   aliases(p_aliases)
 {
-  char *operatorName = nullptr;
+  add_module(p_mod_name, *this);
+  for (const auto &al : aliases) add_alias(al.alias, al.original);
+}
+// clang-format on
 
-  if (operatorCommand)
-    {
-      if (operatorCommand[0] == '-') operatorCommand++;
-      const char *commapos = strchr(operatorCommand, ',');
-      const size_t len = (commapos != nullptr) ? (size_t)(commapos - operatorCommand) : strlen(operatorCommand);
+std::string
+module_t::toString()
+{
+  std::string inp = (get_stream_in_cnt() >= 0) ? std::to_string(get_stream_in_cnt()) : "Arbitrary";
+  std::string out = (get_stream_out_cnt() >= 0) ? std::to_string(get_stream_out_cnt()) : "Output base";
+  std::string restriction = "none";
 
-      operatorName = (char *) malloc(len + 1);
+  if (get_pos_restriction() == OnlyFirst) restriction = "Can only be the first operator";
+  if (get_pos_restriction() == FilesOnly)
+    (restriction != Green("none")) ? restriction = Yellow("Can only use files as input.")
+                                   : restriction += Yellow(", Can only use files as input");
 
-      if (operatorName != nullptr)
-        {
-          memcpy(operatorName, operatorCommand, len);
-        }
-      else{return nullptr;}
-      operatorName[len] = '\0';
-    }
+  std::string desc = "Input: " + inp + ", Ouput: " + out + ", Restricton: " + restriction;
 
-  return operatorName;
+  return desc;
 }
 
+void
+extract_name_and_argument(const std::string &command, std::string &operatorName, std::string &operatorArgument)
+{
+  const std::string delimiter = ",";
+
+  const size_t start = (command[0] == '-') ? 1 : 0;
+
+  size_t len = command.find(delimiter);
+  if (len == std::string::npos)
+    {
+      len = command.size();
+      operatorArgument = "";
+    }
+  else { operatorArgument = command.substr(len + 1, std::string::npos); }
+
+  operatorName = command.substr(start, len - start);
+}
+
+std::string
+extract_operator_name(const std::string &operatorCommand)
+{
+  const std::string delimiter = ",";
+
+  const size_t start = (operatorCommand[0] == '-') ? 1 : 0;
+
+  size_t len = operatorCommand.find(delimiter);
+  if (len == std::string::npos) len = operatorCommand.size();
+
+  std::string oper_name = operatorCommand.substr(start, len - start);
+  return oper_name;
+}
+
+bool
+is_alias(const std::string &operatorName)
+{
+  return (get_aliases().find(operatorName) != get_aliases().end());
+}
+
+std::string
+get_original(const std::string &operatorName)
+{
+  std::string name = extract_operator_name(operatorName);
+  return (is_alias(name) ? get_aliases()[name] : name);
+}
 /**
  * @param a pointer to a string/substring
  * @param b pointer to a string/substring
  * @param alen length of string a
- * @param blen lenght of string b
+ * @param blen length of string b
  * @retval true if a is similar to b
  * @retval false if a is not similar to b
  *
@@ -87,9 +133,9 @@ similar(const char *a, const char *b, unsigned long alen, unsigned long blen)
       b++;
     }
   if (!*a && !*b) return true;
-  /*
-    printf("%d %d %s %s\n", alen, blen, a, b);
-  */
+
+  //  printf("%d %d %s %s\n", alen, blen, a, b);
+
   if (alen >= 2 && blen >= 1 && *a && similar(a + 1, b, alen - 2, blen - 1)) return true;
 
   if (alen >= 1 && blen >= 2 && *b && similar(a, b + 1, alen - 1, blen - 2)) return true;
@@ -106,7 +152,7 @@ similar(const char *a, const char *b, unsigned long alen, unsigned long blen)
  * Wrapper function for #similar() to parse c++ strings to c strings
  */
 static bool
-similar(std::string original, std::string other)
+similar(const std::string &original, const std::string &other)
 {
   return (similar(original.c_str(), other.c_str(), original.size(), other.size()));
 }
@@ -116,12 +162,12 @@ similar(std::string original, std::string other)
  * @retval true if #modules_map contains \p operatorName
  * @retval false if not
  */
-static bool
+bool
 operator_name_exists(const std::string &operatorName)
 {
-  if (modules_map.find(operatorName) != modules_map.end()) return true;
+  if (get_module_map().find(operatorName) != get_module_map().end()) return true;
 
-  if (aliases.find(operatorName) != aliases.end()) return true;
+  if (get_aliases().find(operatorName) != get_aliases().end()) return true;
 
   return false;
 }
@@ -141,7 +187,7 @@ module_map_contains(std::string moduleName)
     }
   else
     {
-      cdoAbort("Module %s not found", moduleName.c_str());
+      cdo_abort("Module %s not found", moduleName);
     }
   return false;
 }
@@ -150,21 +196,21 @@ module_map_contains(std::string moduleName)
  * function for finding similar operator names for the given string
  * @param operatorName operator name to find similar operators for
  * @returns A string with all found names. The string is seqmented into lines
- * with a max lenght of 75 characters
+ * with a max length of 75 characters
  */
-static std::string
-find_similar(const std::string &operatorName)
+std::string
+find_similar_operators(const std::string &operatorName)
 {
   std::string found_similar_operators = "";
-  unsigned long lines = 1;
-  unsigned long line_length = 105;
+  size_t lines = 1;
+  constexpr size_t line_length = 105;
 
   if (operatorName != "")
     {
       // Searching for similar operator names in operator to module map
-      for (const auto &str : modules_map)
+      for (const auto &str : get_module_map())
         {
-          if (similar(stringToLower(operatorName), str.first))
+          if (similar(string_to_lower(operatorName), str.first))
             {
               if (found_similar_operators.size() + str.first.size() > lines * line_length)
                 {
@@ -176,9 +222,9 @@ find_similar(const std::string &operatorName)
             }
         }
       // Searching for similar operator names in aliases to original map
-      for (const auto &str : aliases)
+      for (const auto &str : get_aliases())
         {
-          if (similar(stringToLower(operatorName), str.first))
+          if (similar(string_to_lower(operatorName), str.first))
             {
               if (found_similar_operators.size() + str.first.size() > lines * line_length)
                 {
@@ -203,46 +249,33 @@ find_similar(const std::string &operatorName)
 
  * Checks if \p operatorName is not a file.
 
- * If no matching operator is found checks for similar operators using
- find_similar().
+ * If no matching operator is found checks for similar operators using find_similar_operators().
  *
  *  \note If \p operatorName is a file name the program will exit.
  */
 static bool
 check_operator(const std::string &operatorName)
 {
-  if (operator_name_exists(operatorName))
-    {
-      return true;
-    }
-  else if (operatorName == "")
-    {
-      cdoAbort("Operator name missing!");
-    }
+  if (operator_name_exists(operatorName)) { return true; }
+  else if (operatorName == "") { cdo_abort("Operator name missing!"); }
   else
     {
       // Checking if the operatorname is an existing file name
-      FILE *fp = fopen(operatorName.c_str(), "r");
+      auto fp = std::fopen(operatorName.c_str(), "r");
       if (fp)
         {
-          fclose(fp);
+          std::fclose(fp);
           fprintf(stderr, "Use commandline option -h for help.\n");
-          cdoAbort("Operator missing, %s is a file on disk!", operatorName.c_str());
+          cdo_abort("Operator missing, %s is a file on disk!", operatorName);
         }
       // Operator is no filename
       // Checking for similar operators
       fprintf(stderr, "Operator >%s< not found!\n", operatorName.c_str());
       fprintf(stderr, "Similar operators are:\n");
-      std::string found_similar_operators = find_similar(operatorName);
+      const auto found_similar_operators = find_similar_operators(operatorName);
 
-      if (found_similar_operators.size() > 0)
-        {
-          std::cerr << found_similar_operators << std::endl;
-        }
-      else
-        {
-          fprintf(stderr, "(not found)\n");
-        }
+      if (found_similar_operators.size() > 0) { std::cerr << found_similar_operators << std::endl; }
+      else { fprintf(stderr, "(not found)\n"); }
 
       exit(EXIT_FAILURE);
     }
@@ -251,24 +284,21 @@ check_operator(const std::string &operatorName)
 }
 
 void
-registerOperatorsFromModules()
+register_operators(const std::string &mod_name, const module_t &mod)
 {
-  for (auto &x : modules)
+  for (const auto &y : mod.operators)
     {
-      for (auto y : x.second.operators)
-        {
-          if (!operator_name_exists(y))
-            {
-              modules_map[y] = x.first;
-            }
-          else
-            {
-              cdoAbort("Tried to add operator but the operator name already exists");
-            }
-        }
+      if (!operator_name_exists(y)) { get_module_map()[y] = mod_name; }
+      else { std::cout << "Tried to add operator but the operator name already exists" << std::endl; }
     }
 }
 
+void
+register_operators_from_modules()
+{
+  for (const auto &x : get_modules()) { register_operators(x.first, x.second); }
+}
+
 /***
  * Adds a module and its operators to cdo.
  * Adds the module to modules
@@ -277,28 +307,19 @@ registerOperatorsFromModules()
  * @note: if an error happens while adding the new module cdo will exit.
  */
 void
-add_module(const std::string &module_name, const module_t new_module)
+add_module(const std::string &module_name, const module_t &new_module)
 {
-  if (modules.find(module_name) == modules.end())
+  if (get_modules().find(module_name) == get_modules().end())
     {
-      modules[module_name] = new_module;
-      for (const std::string &operatorName : new_module.operators)
+      get_modules()[module_name] = new_module;
+      for (const auto &operatorName : new_module.operators)
         {
           // if the operator name is not already in the map or in the aliases
-          if (!operator_name_exists(operatorName))
-            {
-              modules_map[operatorName] = module_name;
-            }
-          else
-            {
-              cdoAbort("Tried to add operator but the operator name already exists");
-            }
+          if (!operator_name_exists(operatorName)) { get_module_map()[operatorName] = module_name; }
+          else { cdo_abort("Tried to add operator but the operator name already exists"); }
         }
     }
-  else
-    {
-      cdoAbort("Module %s name already exists", module_name.c_str());
-    }
+  else { cdo_abort("Module %s name already exists", module_name); }
 }
 
 /**
@@ -309,28 +330,28 @@ add_module(const std::string &module_name, const module_t new_module)
 int
 add_alias(const std::string &alias, const std::string &original)
 {
-  auto iter_original = modules_map.find(original);
-  auto iter_alias = aliases.find(alias);
+  auto iter_original = get_module_map().find(original);
+  auto iter_alias = get_aliases().find(alias);
 
-  if (iter_alias != aliases.end())
+  if (iter_alias != get_aliases().end())
     {
-      printf("WARNING: alias %s could not be added: it already exists", alias.c_str());
+      printf("WARNING: alias %s could not be added: it already exists\n", alias.c_str());
       return -1;
     }
 
-  if (iter_original == modules_map.end())
+  if (iter_original == get_module_map().end())
     {
-      printf("ERROR: alias %s could not be added: operator %s does not exist", alias.c_str(), original.c_str());
+      printf("ERROR: alias %s could not be added: operator %s does not exist\n", alias.c_str(), original.c_str());
       return -2;
     }
 
-  if (modules_map.find(alias) != modules_map.end())
+  if (get_module_map().find(alias) != get_module_map().end())
     {
-      printf("ERROR alias %s could not be added: alias name already exists as an operator", alias.c_str());
+      printf("ERROR alias %s could not be added: alias name already exists as an operator\n", alias.c_str());
       return -3;
     }
 
-  aliases[alias] = original;
+  get_aliases()[alias] = original;
 
   return 0;
 }
@@ -343,26 +364,14 @@ add_alias(const std::string &alias, const std::string &original)
  * return value:
  *      std::string -> name of the operators module
  */
-static std::string
+std::string
 get_module_name_to(const std::string &operatorName)
 {
   // if not true the programm will exit in function check_operator
   if (check_operator(operatorName))
     {
-      if (modules_map.count(operatorName) > 0)
-        {
-          return modules_map[operatorName];
-        }
-      else if (aliases.count(operatorName) > 0)
-        {
-          return modules_map[aliases[operatorName]];
-        }
-    }
-  else
-    {
-      // TODO: see if depricated since no operator can be added without a
-      // module
-      cdoAbort("Module for >%s< not installed", operatorName.c_str());
+      if (get_module_map().count(operatorName) > 0) { return get_module_map()[operatorName]; }
+      else if (get_aliases().count(operatorName) > 0) { return get_module_map()[get_aliases()[operatorName]]; }
     }
   // Only to quell the warning. Function should never reach this.
   return "";
@@ -376,23 +385,23 @@ get_module_name_to(const std::string &operatorName)
  * @returns the function of the #module_t of \a operatorName
  */
 /* not used
-void *(*operatorModule(const char *operatorName))(void *)
+void *(*operatorModule(const std::string &operatorName))(void *)
 {
   std::string operator_name = std::string(operatorName);
-  return modules[get_module_name_to(operator_name)].func;
+  return get_modules()[get_module_name_to(operatorName)].func;
 }
 */
 
 /***
  * returns help for given operator name
- * if there is no help a empty vector will be returned
+ * if there is no help nullptr will be returned
  * @param operatorName operator name
  * @return vector of strings containing the help
  */
-std::vector<std::string>
-operatorHelp(const std::string &operatorName)
+const char **
+operator_help(const std::string &operatorName)
 {
-  return modules[get_module_name_to(operatorName)].help;
+  return get_modules()[get_module_name_to(get_original(operatorName))].help;
 }
 
 /***
@@ -406,7 +415,7 @@ int
 operatorStreamInCnt(const char *operatorName)
 {
   std::string operator_name = std::string(operatorName);
-  return modules[get_module_name_to(operator_name)].streamInCnt;
+  return get_modules()[get_module_name_to(operator_name)].streamInCnt;
 }
 */
 /***
@@ -421,7 +430,7 @@ int
 operatorStreamOutCnt(const char *operatorName)
 {
   std::string operator_name = std::string(operatorName);
-  return modules[get_module_name_to(operator_name)].streamOutCnt;
+  return get_modules()[get_module_name_to(operator_name)].streamOutCnt;
 }
 */
 /***
@@ -430,10 +439,9 @@ operatorStreamOutCnt(const char *operatorName)
  * @reval short
  */
 int
-operatorStreamNumber(const char *operatorName)
+operator_stream_number(std::string &p_operatorName)
 {
-  std::string operator_name = std::string(operatorName);
-  return modules[get_module_name_to(operator_name)].number;
+  return get_modules()[get_module_name_to(p_operatorName)].get_number();
 }
 
 /***
@@ -441,23 +449,17 @@ operatorStreamNumber(const char *operatorName)
  * @return a sorted std::vector containing all operator names and aliases
  * excluding all operators which modules are marked as internal
  */
-static std::vector<std::string>
+std::vector<std::string>
 get_sorted_operator_name_list()
 {
-
   std::vector<std::string> names;
-  for (const std::pair<std::string, std::string> &operator_module_names_pair : modules_map)
+  for (const auto &operator_module_names_pair : get_module_map())
     {
-      if (modules[operator_module_names_pair.second].mode == 1)
-        {
-          names.push_back(operator_module_names_pair.first);
-        }
+      if (get_modules()[operator_module_names_pair.second].get_mode() == 1) { names.push_back(operator_module_names_pair.first); }
     }
   // adding operators names from alias_map
-  for (const std::pair<std::string, std::string> &alias : aliases)
-    {
-      names.push_back(alias.first);
-    }
+  for (const auto &alias : get_aliases()) { names.push_back(alias.first); }
+
   std::sort(names.begin(), names.end());
 
   return names;
@@ -467,23 +469,25 @@ std::vector<std::string>
 get_no_output_operator_list()
 {
   std::vector<std::string> names;
-  for (const std::pair<std::string, std::string> &operator_module_names_pair : modules_map)
+  for (const auto &operator_module_names_pair : get_module_map())
     {
-      if (modules[operator_module_names_pair.second].mode == 1 && modules[operator_module_names_pair.second].streamOutCnt == 0)
+      if (get_modules()[operator_module_names_pair.second].get_mode() == 1
+          && get_modules()[operator_module_names_pair.second].get_stream_out_cnt() == 0)
         {
           names.push_back(operator_module_names_pair.first);
         }
     }
   // adding operators names from alias_map
-  std::string original;
-  for (const std::pair<std::string, std::string> &alias : aliases)
+  for (const auto &alias : get_aliases())
     {
-      original = alias.second;
-      if (modules[modules_map[original]].mode == 1 && modules[modules_map[original]].streamOutCnt == 0)
+      const auto &original = alias.second;
+      if (get_modules()[get_module_map()[original]].get_mode() == 1
+          && get_modules()[get_module_map()[original]].get_stream_out_cnt() == 0)
         {
           names.push_back(alias.first);
         }
     }
+
   std::sort(names.begin(), names.end());
 
   return names;
@@ -514,109 +518,31 @@ operatorPrintAll(void)
   std::cerr << std::endl;
 }
 
-#ifdef CUSTOM_MODULES
-/***
-  loads all custom modules in a specified folder
-  @param folder_path custom modules folder
-*/
-void
-load_custom_modules(std::string folder_path)
-{
-  DIR *dir = opendir(folder_path.c_str());
-  std::string file_path;
-  std::regex library_regex("(.*\\.so)");
-  if (dir != nullptr)
-    {
-      struct dirent *ent = readdir(dir);
-      while (ent != nullptr)
-        {
-          if (std::regex_match(ent->d_name, library_regex))
-            {
-              file_path = folder_path + "/" + ent->d_name;
-              load_custom_module(file_path);
-            }
-          ent = readdir(dir);
-        }
-    }
-  else
-    {
-      std::cerr << "Could not find " << folder_path << "for loading custom modules" << std::endl;
-    }
-}
-
-/***
- * Loads a custom module from given path.
- * Modules must contain a (TODO: rename function) init_custom_module function
- * Program exits if a module could not be loaded.
- * @param module file path
- */
-void
-load_custom_module(std::string file_path)
-{
-  void (*init_custom_module)();
-  void *lib_handle = dlopen(file_path.c_str(), RTLD_LAZY);
-  custom_modules_lib_handles.push_back(lib_handle);
-  if (!lib_handle)
-    {
-      std::cerr << "Cannot open library: " << dlerror() << std::endl;
-      return;
-    }
-
-  dlerror();
-  init_custom_module = (void (*)()) dlsym(lib_handle, "init_custom_module");
-  const char *dlsym_error = dlerror();
-
-  if (dlsym_error)
-    {
-      std::cerr << "Cannot load symbol 'init_custom_module': " << dlsym_error << std::endl;
-      dlclose(lib_handle);
-      return;
-    }
-
-  init_custom_module();
-  std::cout << "loaded custom module from '" << file_path << "'" << std::endl;
-}
-
-/***
-  closes the handles for the loaded custum modules
-*/
-void
-close_library_handles()
-{
-  for (unsigned long i = 0; i < custom_modules_lib_handles.size(); i++)
-    {
-      dlclose(custom_modules_lib_handles[i]);
-    }
-}
-#endif
-
-// helper function for setting the spacing in operatorPrintList
+// helper function for setting the spacing in operator_print_list
 static std::string
 get_spacing_for(int p_space, const std::string &str)
 {
   std::string spacing = "";
-  for (int i = str.size(); i <= p_space; i++) spacing += " ";
+  for (int i = str.size(); i <= p_space; ++i) spacing += " ";
   return spacing;
 }
 
 static std::string
-get_operator_description(const std::string &p_current_op_name, const std::vector<std::string> &help)
+get_operator_description(const std::string &p_current_op_name, const char **help)
 {
   std::string description = "";
-  unsigned long cur_help_idx;
-  bool help_contains_name;
+  unsigned long cur_help_idx = 0;
   std::string line;
   unsigned long operator_section = 0;
 
-  cur_help_idx = 0;
   // search for operator section
-  while (operator_section == 0 && cur_help_idx < help.size() - 1)
+  size_t help_size = 0;
+  while (help[help_size]) help_size++;
+  if (!help_size) return description;
+  while (operator_section == 0 && cur_help_idx < help_size - 1)
     {
       line = help[++cur_help_idx];
-      if (line.find("OPERATORS") != std::string::npos)
-        {
-          operator_section = cur_help_idx;
-        }
+      if (line.find("OPERATORS") != std::string::npos) { operator_section = cur_help_idx; }
     }
   // if no operator section is found
   if (operator_section == 0)
@@ -624,30 +550,24 @@ get_operator_description(const std::string &p_current_op_name, const std::vector
       cur_help_idx = 0;
       line = help[0];
       std::string name_section = help[0];
-      help_contains_name = false;
+      bool help_contains_name = false;
       // search for the operator name in the description
       while (!line.empty())
         {
           line = help[++cur_help_idx];
-          if (line.find(p_current_op_name) != std::string::npos)
-            {
-              help_contains_name = true;
-            }
+          if (line.find(p_current_op_name) != std::string::npos) { help_contains_name = true; }
           name_section += line;
         }
       // if the name was found save description for later use
-      if (help_contains_name)
-        {
-          description = name_section.substr(name_section.find_first_of('-') + 2, name_section.size());
-        }
+      if (help_contains_name) { description = name_section.substr(name_section.find_first_of('-') + 2, name_section.size()); }
     }
   else
     {
-      line = help.at(++operator_section);
+      line = help[++operator_section];
       // search the operator section for current operator line
-      while (line.find(p_current_op_name + " ") == std::string::npos && !line.empty() && operator_section < help.size() - 1)
+      while (line.find(p_current_op_name + " ") == std::string::npos && !line.empty() && operator_section < help_size - 1)
         {
-          line = help.at(++operator_section);
+          line = help[++operator_section];
         }
       // if operator line found save description for later use
       if (!line.empty() && line.find("    " + p_current_op_name + " ") != std::string::npos)
@@ -669,70 +589,61 @@ get_operator_description(const std::string &p_current_op_name, const std::vector
  * If the operator is not documented the description is empty
  */
 void
-operatorPrintList(bool print_no_output)
+operator_print_list(bool print_no_output)
 {
-  std::vector<std::string> output_list;
-  if (print_no_output)
-    {
-      output_list = get_no_output_operator_list();
-    }
-  else
-    {
-      output_list = get_sorted_operator_name_list();
-    }
+  std::vector<std::string> output_list = print_no_output ? get_no_output_operator_list() : get_sorted_operator_name_list();
 
   auto list_length = output_list.size();
-  module_t *current_module;
 
   // help variables
 
   for (size_t out_list_idx = 0; out_list_idx < list_length; out_list_idx++)
     {
-      std::string current_op_name = output_list[out_list_idx];
-      current_module = &modules[get_module_name_to(current_op_name)];
-      if (aliases.find(current_op_name) != aliases.end())
+      auto &current_op_name = output_list[out_list_idx];
+      const auto *current_module = &get_modules()[get_module_name_to(current_op_name)];
+      if (get_aliases().find(current_op_name) != get_aliases().end())
         {
-          output_list[out_list_idx] += std::string(get_spacing_for(16, current_op_name) + "--> " + aliases[current_op_name]);
+          output_list[out_list_idx] += get_spacing_for(16, current_op_name) + "--> " + get_aliases()[current_op_name];
         }
-      else if (!current_module->help.empty())
+      else if (current_module->help)
         {
           // add spaceing and saving output line to the output list
-          std::string description = get_operator_description(current_op_name, current_module->help);
+          auto description = get_operator_description(current_op_name, current_module->help);
           output_list[out_list_idx] += get_spacing_for(16, current_op_name) + description;
         }
-      std::string in_out_info
-          = "(" + std::to_string(current_module->streamInCnt) + "|" + std::to_string(current_module->streamOutCnt) + ")";
+      std::string in_out_info = " (" + std::to_string(current_module->get_stream_in_cnt()) + "|"
+                                + std::to_string(current_module->get_stream_out_cnt()) + ")";
       output_list[out_list_idx] += get_spacing_for(90, output_list[out_list_idx]) + in_out_info;
     }
   // print generated output list
-  for (const std::string &str : output_list)
-    {
-      std::cout << str << std::endl;
-    }
+  for (const std::string &str : output_list) { std::cout << str << std::endl; }
 }
 
-static bool
-is_alias(const char *operatorName)
+std::map<std::string, module_t>::iterator
+find_module(const std::string &operator_name)
 {
-  return (aliases.find(std::string(operatorName)) != aliases.end());
+  std::string operName = operator_name;
+
+  auto it = get_aliases().find(operName);
+  if (it != get_aliases().end()) { operName = it->second; }
+
+  auto modMapIter = get_module_map().find(operName);
+  auto modIter = get_modules().end();
+  if (!(modMapIter == get_module_map().end())) { modIter = get_modules().find(modMapIter->second); }
+  return modIter;
 }
 
-const char *
-get_original(const char *operatorName)
+std::vector<std::string>
+get_module_operator_names(std::string module_name)
 {
-  const char *name = getOperatorName(operatorName);
-  if (is_alias(name))
-    {
-      std::string opName = aliases[std::string(name)];
-      char *original = (char *) malloc(opName.size() + 1);
-      strcpy(original, opName.c_str());
-      return original;
-    }
-  return name;
+  auto &modules = get_modules();
+  if (std::islower(static_cast<unsigned char>(module_name[0]))) {module_name[0] = std::toupper(module_name[0]); }
+  if (modules.find(module_name) == modules.end()) { return std::vector<std::string>(); }
+  return modules[module_name].operators;
 }
 
 module_t &
-getModule(const std::string &operator_name)
+get_module(const std::string &operator_name)
 {
-  return modules[get_module_name_to(operator_name)];
+  return get_modules()[get_module_name_to(get_original(operator_name))];
 }
