@@ -450,6 +450,9 @@ cdo_healpix_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals, bo
       cdo_abort("%s mapping parameter missing!", projection);
     }
 
+  size_t gridsize = 12 * nside * nside;
+  if (gridsize != nvals) cdo_abort("Not a global healpix grid (nside=%d  numCells=%zu/%zu)!", nside, nvals, gridsize);
+
   auto hpOrder = hp_get_order(order);
   if (hpOrder == HpOrder::Undef) cdo_abort("%s mapping parameter healpix_order=%s unsupported!", projection, order);
 
