@@ -9,7 +9,7 @@
 #include <algorithm>  // std::sort
 
 #include "process_int.h"
-#include "cdo_wtime.h"
+#include "cdo_timer.h"
 #include <mpim_grid.h>
 #include "cdo_options.h"
 #include "remap.h"
@@ -121,7 +121,7 @@ remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
 
   if (srcGrid->rank != 2) cdo_abort("Can't do bicubic interpolation if the source grid is not a regular 2D grid!");
 
-  auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
+  cdo::timer timer;
 
   progress::init();
 
@@ -197,7 +197,7 @@ remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
 
   weight_links_4_to_remap_links(tgtGridSize, weightLinks, rv);
 
-  if (Options::cdoVerbose) cdo_print("%s: %.2f seconds", __func__, cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdo_print("%s: %.2f seconds", __func__, timer.elapsed());
 }  // remap_bicubic_weights
 
 /*
@@ -235,7 +235,7 @@ remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArr
 
   if (srcGrid->rank != 2) cdo_abort("Can't do bicubic interpolation if the source grid is not a regular 2D grid!");
 
-  auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
+  cdo::timer timer;
 
   progress::init();
 
@@ -316,7 +316,7 @@ remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArr
 
   progress::update(0, 1, 1);
 
-  if (Options::cdoVerbose) cdo_print("%s: %.2f seconds", __func__, cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdo_print("%s: %.2f seconds", __func__, timer.elapsed());
 }  // remap_bicubic
 
 void
