@@ -113,10 +113,10 @@ public:
     nlat = _nlat;
     ntr = _ntr;
 
-    const long nsp = (ntr + 1) * (ntr + 2);
-    const long poldim = (nsp / 2) * nlat;
+    long nsp = (ntr + 1) * (ntr + 2);
+    long poldim = (nsp / 2) * nlat;
 
-    const auto numArrays = (polFlag == PolFlag::UV2DV) ? 2 : 1;
+    auto numArrays = (polFlag == PolFlag::UV2DV) ? 2 : 1;
     cdo_print("Allocate %d array%s to hold the Legendre polynoms: size=%ld Bytes", numArrays, numArrays > 1 ? "s" : "", poldim * 8);
 
     if (polFlag == PolFlag::SP2FC) varrayResize(poli, poldim);
@@ -155,7 +155,7 @@ public:
   {
     ntr = _ntr;
 
-    const long dimsp = (ntr + 1) * (ntr + 2);
+    long dimsp = (ntr + 1) * (ntr + 2);
     fdim = dimsp / 2;
 
     f1.resize(fdim);
